/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "pager.h"

#include <KDE/KPushButton>
#include <KDE/KWindowSystem>

#include <QButtonGroup>
#include <QHBoxLayout>
#include <QToolButton>

#include <kdebug.h>

Pager::Pager(QWidget *parent) : QFrame(parent)
{
    setObjectName("Pager");

    m_desktops = new QButtonGroup(this);
    m_pagerLayout = new QHBoxLayout(this);
    m_pagerLayout->setSpacing(0);
    m_pagerLayout->setMargin(0);

    void slotCurrentDesktopChanged (int desktop);
    void slotDesktopNamesChanged ();
    void slotNumberOfDesktopsChanged (int num);

    connect( KWindowSystem::self(), SIGNAL(currentDesktopChanged(int)), this, SLOT(slotCurrentDesktopChanged(int)));
    connect( KWindowSystem::self(), SIGNAL(desktopNamesChanged()), this, SLOT(slotDesktopNamesChanged()));
    connect( KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)), this, SLOT(slotNumberOfDesktopsChanged(int)));
    connect( m_desktops, SIGNAL(buttonClicked(QAbstractButton*)), this, SLOT(slotSetCurrentDesktop(QAbstractButton*)));

    int desktopCount = KWindowSystem::numberOfDesktops( );
    for( int idx = 1; idx  <= desktopCount; ++idx)
    {
        QToolButton *button = new QToolButton(this);
        button->setText(QString::number( idx ) );
        button->setToolTip( KWindowSystem::desktopName(idx) );
        m_pagerLayout->addWidget( button );
        m_desktops->addButton( button, idx );
    }

    if( desktopCount == 1 )
        hide();
    else
        show();
}

void Pager::slotCurrentDesktopChanged (int desktop)
{
    QAbstractButton *button = m_desktops->button( desktop );
    kDebug() << "Changed to desktop" << desktop << "on desktop" <<  KWindowSystem::currentDesktop();
}

void Pager::slotDesktopNamesChanged ()
{
    int buttonCount = m_desktops->buttons().count();
    for( int idx = 1; idx <= buttonCount; ++idx )
    {
        QAbstractButton *button = m_desktops->button( idx );
        if( button )
            button->setToolTip( KWindowSystem::desktopName(idx) );
    }
}

void Pager::slotNumberOfDesktopsChanged (int desktopCount)
{
    qDeleteAll( m_desktops->buttons() );
    for( int idx = 1; idx  <= desktopCount; ++idx)
    {
        QToolButton *button = new QToolButton(this);
        button->setText(QString::number( idx ) );
        button->setToolTip( KWindowSystem::desktopName(idx) );
        m_pagerLayout->addWidget( button );
        m_desktops->addButton( button, idx );
    }

    if( desktopCount == 1 )
        hide();
    else
        show();
}

void Pager::slotSetCurrentDesktop( QAbstractButton *button )
{
    int desktop = m_desktops->id( button );
    KWindowSystem::setCurrentDesktop( desktop );
        kDebug() << "set current desktop" << desktop;
}
