/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "topbar.h"
#include "clock.h"
#include "systemtray.h"
#include "launcherbutton.h"
#include "mediabutton.h"
#include "taskmanagerwidget.h"
#include "pager.h"
#include "launcherbar.h"
#include "desktopbutton.h"

#include <kdebug.h>

#include <QHBoxLayout>
#include <QDesktopWidget>
#include <QX11Info>
#include <QApplication>
#include <QPalette>

#include <KDE/KWindowSystem>

TopBar::TopBar( QWidget *parent ) : QFrame( parent, Qt::FramelessWindowHint)
{
    setObjectName("TopPanel");

    m_clock = new Clock(this);
    m_tray = new SystemTray(this);

    m_button = new LauncherButton(this);

    m_media = new MediaButton(this);

    m_launcher = new LauncherBar(this);

    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->addWidget(m_button);
    layout->addWidget(m_media);
    layout->addWidget(m_launcher, 1);
    layout->addWidget( new QWidget(this), 1); // give some space here...
    layout->addWidget(m_tray);
    layout->addWidget(m_clock);
    layout->setSpacing(0);
    layout->setMargin(0);

    KWindowSystem::setType( winId(), NET::Dock );
    KWindowSystem::setOnAllDesktops( winId(), true );
}

void TopBar::slotDesktopResized ( int screen, int panelSize )
{
    if( screen == QApplication::desktop()->primaryScreen() )
    {
        QRect screenGeometry = QApplication::desktop()->screenGeometry(QApplication::desktop()->primaryScreen());
        int screenWidth = screenGeometry.width();
        setFixedSize( screenWidth, panelSize );
        setGeometry(screenGeometry.x(), 0, screenWidth, panelSize );
        // FIXME: make this work on dual head.
        KWindowSystem::setExtendedStrut( winId(), 0, 0, 0, 
                                                  0, 0, 0, 
                                                  panelSize, screenGeometry.x(), screenGeometry.x() + screenWidth - 1,
                                                  0, 0, 0);
   }
}

BottomBar::BottomBar( QWidget *parent ) : QFrame( parent)
{
    setObjectName("BottomPanel");
    m_tasks = new TaskManagerWidget(this);
    m_desktopButton = new DesktopButton(this);

    m_pager = new Pager(this);

    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->addWidget(m_desktopButton);
    layout->addWidget(m_tasks, 1);
    layout->addWidget(m_pager);

    layout->setSpacing(0);
    layout->setMargin(0);

    KWindowSystem::setType( winId(), NET::Dock );
    KWindowSystem::setOnAllDesktops( winId(), true );
}

void BottomBar::slotDesktopResized ( int screen, int panelSize )
{
    if( screen == QApplication::desktop()->primaryScreen() )
    {
        QRect screenGeometry = QApplication::desktop()->screenGeometry(QApplication::desktop()->primaryScreen());
        int screenWidth = screenGeometry.width();
        int screenHeight = screenGeometry.height();
	setGeometry(screenGeometry.x(), screenHeight - panelSize, screenWidth, panelSize );
        setFixedSize( screenWidth, panelSize );
	// FIXME: make this work on dual head.
        KWindowSystem::setExtendedStrut( winId(), 0, 0, 0,
                                                  0, 0, 0,
                                                  0, 0, 0,
                                                  panelSize, screenGeometry.x(), screenGeometry.x() + screenWidth - 1 );
    }
    m_tasks->setMaximumWidth( width() - m_pager->width() );
}
