//
// C++ Implementation: kcolortreewidget
//
// Description: 
//
//
// Author: Olivier Goffart 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kcolorsconfigtreewidget.h"
#include "kcolorsettingsitem.h"
#include "kcolorsettings.h"
#include <klocale.h>
#include <kdebug.h>
#include <kcolordialog.h>
#include <kconfig.h>

#include <QColor>
#include <QItemDelegate>
#include <QPainter>
#include <kglobalsettings.h>


//BEGIN KColorsConfigTreeWidgetItem
/*
    QListViewItem subclass to display/edit a style, bold/italic is check boxes,
    normal and selected colors are boxes, which will display a color chooser when
    activated.
    The context name for the style will be drawn using the editor default font and
    the chosen colors.
    This widget id designed to handle the default as well as the individual hl style
    lists.
    This widget is designed to work with the KateStyleTreeWidget class exclusively.
    Added by anders, jan 23 2002.
*/
class KColorsConfigTreeWidgetItem : public QTreeWidgetItem
{
  public:
    /*KColorsConfigTreeWidgetItem( QTreeWidget *parent, const QString& role, int idx,
						  const KColorSettingsItem::StyleFlags& mask = 0);
	KColorsConfigTreeWidgetItem( QTreeWidgetItem *parent, const QString& role, int idx,
						  const KColorSettingsItem::StyleFlags& mask = 0);
    ~KColorsConfigTreeWidgetItem() {};*/
	  
	   KColorsConfigTreeWidgetItem( QTreeWidget * parent , const KColorSettingsItem& item , const QString& role, int idx);

    QString role() const { return text(0); };
    int idx() { return m_idx; }
	
	

    virtual QVariant data( int column, int role ) const;
    virtual void setData( int column, int role, const QVariant& value );
    
    QColor textColor() {  return data(1 , Qt::BackgroundColorRole).value<QColor>(); }
    void setTextColor(const QColor& col); 
    QColor bgColor() {  return data(2 , Qt::BackgroundColorRole).value<QColor>(); }
    void setBgColor(const QColor& col);
	
	void setColorSettingsItem(const KColorSettingsItem& item);
	KColorSettingsItem colorSettingsItem() ;

  private:
      int m_idx;
	  KColorSettingsItem m_item;
	  KColorSettingsItem::StyleFlags m_mask;
};


/*KColorsConfigTreeWidgetItem::KColorsConfigTreeWidgetItem( QTreeWidget * parent, const QString & role, int idx, 
											const KColorSettingsItem::StyleFlags& mask )
    : QTreeWidgetItem( parent ), m_idx(idx), m_mask(mask)
{
    setText(0, role);
}

KColorsConfigTreeWidgetItem::KColorsConfigTreeWidgetItem( QTreeWidgetItem * parent, const QString & role, int idx, 
											const KColorSettingsItem::StyleFlags& mask )
    : QTreeWidgetItem( parent ), m_idx(idx), m_mask(mask)
{
    setText(0, role);
}*/

KColorsConfigTreeWidgetItem::KColorsConfigTreeWidgetItem( QTreeWidget * parent, const KColorSettingsItem& item , const QString & role, int idx)
    : QTreeWidgetItem( parent ), m_idx(idx), m_mask(0)
{
	m_mask=~m_mask;
    setText(0, role);
	setColorSettingsItem(item);
}


void KColorsConfigTreeWidgetItem::setTextColor( const QColor & col )
{
    setData( 1 , Qt::BackgroundColorRole , col );
    setData( 0 , Qt::TextColorRole , col );
}

void KColorsConfigTreeWidgetItem::setBgColor( const QColor & col )
{
    setData( 2 , Qt::BackgroundColorRole , col );
    setData( 0 , Qt::BackgroundColorRole , col );
}


void KColorsConfigTreeWidgetItem::setColorSettingsItem( const KColorSettingsItem & item )
{
	m_item=item;
	setTextColor( item.foregroundColor() );
	setBgColor( item.backgroundColor() );
}


KColorSettingsItem  KColorsConfigTreeWidgetItem::colorSettingsItem(  )
{
//	if(m_mask & KColorSettingsItem::ForegroundColor )
		m_item.setForegroundColor( textColor() );
//	if(m_mask & KColorSettingsItem::BackgroundColor )
		m_item.setBackgroundColor( bgColor() );
		
	kDebug() <<  m_mask << "  " << bgColor() << " == " <<m_item.backgroundColor()  << endl ;

	return m_item;
}




QVariant KColorsConfigTreeWidgetItem::data( int column, int role ) const
{
//     if (column == 1) {
//         switch (role) {
//             case Qt::TextColorRole:
//                 if (style()->hasProperty(QTextFormat::ForegroundBrush))
//                     return style()->foreground().color();
//                 break;
// 
//             case Qt::BackgroundColorRole:
//                 if (style()->hasProperty(QTextFormat::BackgroundBrush))
//                     return style()->background().color();
//                 break;
// 
//             case Qt::FontRole:
//                 return style()->font();
//                 break;
//         }
//     }
// 
//     if (role == Qt::DisplayRole) {
//         switch (column) {
//             case Foreground:
//                 return style()->foreground();
//             case SelectedForeground:
//                 return style()->selectedForeground();
//             case Background:
//                 return style()->background();
//             case SelectedBackground:
//                 return style()->selectedBackground();
//         }
//     }

    return QTreeWidgetItem::data(column, role);
}

void KColorsConfigTreeWidgetItem::setData( int column, int role, const QVariant& value )
{
    QTreeWidgetItem::setData(column, role, value);
}


//END


//BEGIN KColorTreeDelegate
class KColorTreeDelegate : public QItemDelegate
{
    public:
        KColorTreeDelegate(QWidget* widget) : m_widget(widget) {};

        virtual void paint( QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index ) const;

    private:
        QWidget* m_widget;
};


void KColorTreeDelegate::paint( QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index ) const
{
    if (index.column() == 0)
        return QItemDelegate::paint(painter, option, index);

    QBrush brush =  qVariantValue<QBrush>(index.data(Qt::BackgroundColorRole));

    QStyleOptionButton opt;
    opt.rect = option.rect;
    opt.palette = m_widget->palette();

    if(brush != QBrush())
    {
        m_widget->style()->drawControl(QStyle::CE_PushButton, &opt, painter, m_widget);
        painter->fillRect(m_widget->style()->subElementRect(QStyle::SE_PushButtonContents, &opt,m_widget), brush);
    }
    else
    {
        painter->fillRect(option.rect,QBrush(KGlobalSettings::textColor(), Qt::FDiagPattern) );
    }

//     if (!set) {
//         opt.text = i18nc("No text or background colour set", "None set");
//         //brush = Qt::white;
//         brush = QBrush(Qt::white, Qt::DiagCrossPattern);
//     }

}

//END



/*#define KCTW_COLUMN_NAME 0
#define KCTW_COLUMN_FOREGROUND 1
#define KCTW_COLUMN_BACKGROUND 2
#define KCTW_COLUMN_BOLD 3
#define KCTW_COLUMN_ITALIC 4
#define KCTW_COLUMN_UNDERLINE 5
#define KCTW_COLUMN_STRIKE 6
#define KCTW_COLUMN_FONT 7*/

KColorsConfigTreeWidget::KColorsConfigTreeWidget(QWidget *parent)
  : QTreeWidget(parent)
{
  setItemDelegate(new KColorTreeDelegate(this));

  setColumnCount( 3 );
  QStringList headers;
  /*headers << i18n("Role") << i18n("Text") << i18n("Background")
	//	  << i18n("Selected") << i18n("Background Selected") 
		  << QString() << QString() << QString() << QString() << 
		  << i18n("Font");*/
  headers << i18n("Role") << i18n("Text") << i18n("Background");
  setHeaderLabels(headers);
}


KColorsConfigTreeWidget::~KColorsConfigTreeWidget()
{
}


void KColorsConfigTreeWidget::fill(const QString &appname)
{
	clear();
	m_items.clear();
	m_appname=appname;
	KColorSettings settings(appname);
	
	KConfig config("data" , m_appname + '/' + m_appname + ".kcolorsrc" );
	int f=0;
	foreach(QString key , config.groupList() )
	{
		KColorsConfigTreeWidgetItem* i =  new KColorsConfigTreeWidgetItem( this, settings.color( key ) , key, f);
		m_items[ f ] = i ;
		f++;
	}
	

}


void KColorsConfigTreeWidget::save()
{
	if(m_appname.isEmpty())
		return;
	KColorSettings settings(m_appname);
	KConfig config("data" , m_appname + '/' + m_appname + ".kcolorsrc" );
	int f=0;
	foreach(QString key , config.groupList() )
	{
		settings.setColor( key , m_items[ f ]->colorSettingsItem() );
		f++;
	}
	
}

/*void KColorsConfigTreeWidget::setColorSettingsItem( int idx, const KColorSettingsItem & item )
{
	if(m_items.contains(idx))
		m_items[idx]->setColorSettingsItem(item);
	emit colorChanged( idx , item );
}


KColorSettingsItem KColorsConfigTreeWidget::colorSettingsItem( int idx )
{
	if(m_items.contains(idx))
		return m_items[idx]->colorSettingsItem();
	
	return KColorSettingsItem();
}*/


bool KColorsConfigTreeWidget::edit( const QModelIndex & index, EditTrigger trigger, QEvent * event )
{
    KColorsConfigTreeWidgetItem *i = dynamic_cast<KColorsConfigTreeWidgetItem*>(itemFromIndex(index));
    if (!i || index.column() == 0)
        return QTreeWidget::edit(index, trigger, event);
    
  /*  if( (index.column() == 1 && i->fgIndex() == -1)  ||
            (index.column() == 2 && i->bgIndex() == -1) )
        return QTreeWidget::edit(index, trigger, event);*/

    switch (trigger) {
        case QAbstractItemView::DoubleClicked:
        case QAbstractItemView::SelectedClicked:
        case QAbstractItemView::EditKeyPressed:
        {
            QColor c;
            QColor d;
            if ( KColorDialog::getColor( c, d, this ) != QDialog::Accepted) 
                return false;
            if( index.column() == 1) 
				i->setTextColor( c );
			else if( index.column() == 2) 
				i->setBgColor( c);
			emit colorChanged( i->idx() , i->colorSettingsItem() );
            return false;
        }
        default:
            return QTreeWidget::edit(index, trigger, event);
    }

}



#include "kcolorsconfigtreewidget.moc"


