/*
    This file is part of KDE.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "installerrpm.h"

#include "entry.h"
#include "engine.h"

#include "kupdateiface_stub.h"

#include <kmessagebox.h>
#include <klocale.h>
#include <kprocess.h>
#include <kdebug.h>

#include <qwidget.h>

InstallerRpm::InstallerRpm( Engine *e, QWidget *widget )
  : Installer( e, widget )
{
  kdDebug() << "InstallerRpm()" << endl;

  connect( e, SIGNAL( downloadFinished( const QString & ) ),
    SLOT( slotDownloadFinished( const QString & ) ) );
  connect( e, SIGNAL( downloadFailed( const QString & ) ),
    SLOT( slotDownloadFailed( const QString & ) ) );
}

void InstallerRpm::doInstall( KNS::Entry *entry )
{
  engine()->download( entry );
}

void InstallerRpm::slotDownloadFinished( const QString &path )
{
  kdDebug() << "slotDownloadFinished(): " << path << endl;

  KProcess *proc = new KProcess;

  *proc << "kdesu";
  *proc << "-t";  // Enable terminal output
  *proc << "rpm -U " + path;
  
  connect( proc, SIGNAL( processExited( KProcess * ) ),
    SLOT( slotProcessExited( KProcess * ) ) );
  connect( proc, SIGNAL( receivedStdout( KProcess *, char *, int ) ),
    SLOT( slotReceivedStdout( KProcess *, char *, int ) ) );
  connect( proc, SIGNAL( receivedStderr( KProcess *, char *, int ) ),
    SLOT( slotReceivedStderr( KProcess *, char *, int ) ) );

  mStdout = QString::null;
  mStderr = QString::null;

  proc->start( KProcess::NotifyOnExit, KProcess::AllOutput );
}

void InstallerRpm::slotDownloadFailed( const QString &msg )
{
  KMessageBox::sorry( widget(), msg, i18n("Download failed") );
}

void InstallerRpm::slotProcessExited( KProcess *proc )
{
  kdDebug() << "InstallerRpm::slotProcessExited()" << endl;

  if ( proc->normalExit() && proc->exitStatus() == 0 ) {
    finishInstallation();
  } else {
    QString msg = mStdout + mStderr;
    emit installationFailed( msg );
  }

  delete proc;
}

void InstallerRpm::slotReceivedStdout( KProcess *proc, char *buffer,
  int buflen )
{
  Q_UNUSED( proc );
  QCString str( buffer, buflen );
  kdDebug() << "STDOUT: " << str << endl;

  mStdout += str;
}

void InstallerRpm::slotReceivedStderr( KProcess *proc, char *buffer,
 int buflen )
{
  Q_UNUSED( proc );
  QCString str( buffer, buflen );
  kdDebug() << "STDERR: " << str << endl;

  mStderr += str;
}

#include "installerrpm.moc"
