/*  This file is part of the KDE libraries

    Copyright (c) 2005 Ramakrishna R <coolrama@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/



#include "kio_klb.h"

#include <stdlib.h>
#include <qstring.h>
#include <kinstance.h>
#include <kurl.h>
#include <lbapi.h>
#include <qstringlist.h>
#include <kdebug.h>

#define NO_LABEL_SELECTED 0
#define LABEL_SELECTED 1

using namespace KIO;

KlbProtocol::KlbProtocol(const QCString &pool_socket, const QCString &app_socket)
    : SlaveBase("klb", pool_socket, app_socket)
{
}

KlbProtocol::~KlbProtocol()
{
}

void KlbProtocol::createLabel(const KURL &url)
{
	kdDebug() << " Entered createLabel() " << endl;
	QString name = url.queryItem("name");
    
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	t->createLabel(name);
	
	kdDebug()<<"created label " <<endl;
        
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> Created Label " << name << " successfully ";
	os << "<br><br>Go to <A HREF=klb:/><b> KLB HOME</b> </A> </center>";
	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;

}


void KlbProtocol::createForm(const KURL &url)
{
	kdDebug() << " Entered createForm() " << endl;
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);
	
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> <h2><u> Create Label</u> </h2></center><br><br> "; 
	os << "<center> <form name=\"create\" action=\"klb:/create\" method=\"get\"> Label Name <input type = \"text\" name=\"name\"> <input type=\"submit\" value=\"Create\" > </form> </center>";
//	os << "<br><br>Go to <A HREF=klb:/><b> KLB HOME</b> </A> </center>";
	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;
}


void KlbProtocol::applyForm(const KURL &url)
{
	kdDebug() << " Entered applyForm() " << endl;
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);
	
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> <h2><u> Apply Label</u> </h2></center><br><br> "; 

	os <<" <form name=\"apply\" action=\"klb:/apply\" method=\"get\"> Label Name ";
	os << "<select name=\"name\"> ";
	QStringList allLabels;
	KLB::LabelManager * tl = new KLB::LabelManager();
	tl->getAllLabels(allLabels);
	for ( QStringList::Iterator it = allLabels.begin(); it != allLabels.end(); ++it )
        {
        	kdDebug()<<" retrieving individual labels "<<endl;
		os << " <option>"<< *it << " </option>";
        }
	os << "</select>";

	os << "&nbsp; &nbsp; &nbsp;  ";

 	os << "<input name=\"path\" type=\"file\" />";
	os << "<input type=\"submit\" value=\"Apply\">";

	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;
}



void KlbProtocol::applyLabel(const KURL &url)
{
	kdDebug() << " Entered applyLabel() " << endl;
	QString name = url.queryItem("name");
	QString path = url.queryItem("path");
    
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	t->applyLabel(name,path);
	
	kdDebug()<<"applied label " <<endl;
        
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> Applied Label " << name << " successfully to "<<path;
	os << "<br><br>Go to <A HREF=klb:/><b> KLB HOME</b> </A> </center>";
	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;

}

void KlbProtocol::labelCombination(const KURL & url)
{
	kdDebug() << " Entered labelCombination() " << endl;
	QString searchString = url.queryItem("searchstring");


    QByteArray output;
    QTextStream os(output, IO_WriteOnly);
    os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	QStringList* allPaths = new QStringList();
	t->searchLabels(*allPaths, searchString);
	
	kdDebug()<<"got all paths into qstring " <<endl;
	 QStringList tempPathList;

        os << "<html><head><title>K LABEL BROWSER</title></head><body>";
        kdDebug()<<" html head written "<<endl;

	QStringList allLabels;
	KLB::LabelManager * tl = new KLB::LabelManager();
	tl->getAllLabels(allLabels);
	
	kdDebug()<<"got all labels into qstringlist " <<endl;
	
	os << " <u><center><h3><font color=#0000dd>KLABEL BROWSER</font></h3></center></u>";
	os << "<table width=\"764\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\"> ";
	os << " <tr valign=\"top\">";
	os << "<td width=\"30\">&nbsp;</td>"; 
	os << "<td width=\"200\">"<<endl ;
	os << "<br><br>";
	
	os << "<h4>All Labels</h4><br>";
	for ( QStringList::Iterator it = allLabels.begin(); it != allLabels.end(); ++it )
        {
        	kdDebug()<<" retrieving individual labels "<<endl;
		os<< "<a href= klb:/view?searchstring=names=\"";
		os<<  *it <<"\"";

	//	os<< "<a href= \"klb:/";
	//	os<<  *it ;
		os << " ><font color=#007700>";
		os << *it << "</font></a><br>";
        }

	
	os<< "</td>";
	os<< "<td width=\"10\">&nbsp;</td> <td width=\"744\"> <br><br> ";
	os<< "Search string " << searchString << " gave the following results<br><br> "<<endl;
        for ( QStringList::Iterator it = allPaths->begin(); it != allPaths->end(); ++it )
        {
        	kdDebug()<<" retrieving individual paths "<<endl;
		QString tempPath = *it;
		kdDebug()<<"tempPath"<< tempPath <<endl;
		tempPathList = tempPathList.split("/",tempPath);
		QString fileName =*tempPathList.at(tempPathList.size()-1);
		kdDebug()<<"filename :"<< fileName <<endl;
		os << "<A HREF=\"" << *it << "\">"<<fileName<<"</A>"<<endl; 
		os << "<br>";
       }
	
	kdDebug()<<"finished retrieving individual paths"<<endl;
	
	os << "</td> </tr> </table>";
	os<< "</body></html>"<<endl;

    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;
}

void KlbProtocol::deleteLabel(const KURL &url)
{
	kdDebug() << " Entered deleteLabel() " << endl;
	QString name = url.queryItem("name");
    
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	t->deleteLabel(name);
	
	kdDebug()<<"deleted label " <<endl;
        
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> Deleted Label " << name << " successfully ";
	os << "<br><br>Go to <A HREF=klb:/><b> KLB HOME</b> </A> </center>";
	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;

}

void KlbProtocol::renameLabel(const KURL &url)
{
	kdDebug() << " Entered renameLabel() " << endl;
	QString oldname = url.queryItem("oldname");
	QString newname = url.queryItem("newname");
    
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	t->renameLabel(oldname,newname);
	
	kdDebug()<<"renamed label " <<endl;
        
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> Renamed Label " << oldname << " to "<< newname;
	os << "<br><br>Go to <A HREF=klb:/><b> KLB HOME</b> </A> </center>";
	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;

}

void KlbProtocol::removeLabel(const KURL &url)
{
	kdDebug() << " Entered removeLabel() " << endl;
	QString name = url.queryItem("name");
	QString path = url.queryItem("path");
    
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	t->removeLabel(name,path);
	
	kdDebug()<<"removed label " << "from path "<< path<<endl;
        
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> Removed Label " << name << " from path "<< path;
	os << "<br><br>Go to <A HREF=klb:/><b> KLB HOME</b> </A> </center>";
	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;

}

void  KlbProtocol::showMain(const KURL& url)
{
	QString labelName = url.path();
	int length = labelName.length();
	labelName = labelName.right(length-1);

    QByteArray output;
    QTextStream os(output, IO_WriteOnly);
    os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	QStringList allPaths;
	t->getPaths(labelName,allPaths);
	
	kdDebug()<<"got all paths into qstring " <<endl;
	 QStringList tempPathList;

        os << "<html><head><title>K LABEL BROWSER</title></head><body>";
        kdDebug()<<" html head written "<<endl;

	QStringList allLabels;
	KLB::LabelManager * tl = new KLB::LabelManager();
	tl->getAllLabels(allLabels);
	
	kdDebug()<<"got all labels into qstringlist " <<endl;
	
	os << " <u><center><h3><font color=#0000dd>KLABEL BROWSER</font></h3></center></u>";
	os << "<table width=\"764\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\"> ";
	os << " <tr valign=\"top\">";
	os << "<td width=\"30\">&nbsp;</td>"; 
	os << "<td width=\"200\">"<<endl ;
	os << "<br><br>";
	
	os << "<h4>All Labels</h4><br>";
	for ( QStringList::Iterator it = allLabels.begin(); it != allLabels.end(); ++it )
        {
        	kdDebug()<<" retrieving individual labels "<<endl;
		os<< "<a href= klb:/view?searchstring=names=\"";
		os<<  *it <<"\"";
		os << "><font color=#007700>";
		os << *it << "</font></a><br>";
        }

	
	os<< "</td>";
	os<< "<td width=\"10\">&nbsp;</td> <td width=\"744\"> <br><br> ";
	os<< labelName << " applied to following links <br>";
        for ( QStringList::Iterator it = allPaths.begin(); it != allPaths.end(); ++it )
        {
        	kdDebug()<<" retrieving individual paths "<<endl;
		QString tempPath = *it;
		kdDebug()<<"tempPath"<< tempPath <<endl;
		tempPathList = tempPathList.split("/",tempPath);
		QString fileName =*tempPathList.at(tempPathList.size()-1);
		kdDebug()<<"filename :"<< fileName <<endl;
		os << "<A HREF=\"" << *it << "\">"<<fileName<<"</A>"<<endl; 
		os << "<br>";
       }
	
	kdDebug()<<"finished retrieving individual paths"<<endl;
	
	os << "</td> </tr> </table>";
	os<< "</body></html>"<<endl;

    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;

}

void KlbProtocol::addKeywords(const KURL &url)
{
	kdDebug() << " Entered addKeywords() " << endl;
	QString name = url.queryItem("name");
	QString keywords = url.queryItem("keywords");
	
	QStringList lt;
	lt = lt.split(",",keywords);
    
 	QByteArray output;
	QTextStream os(output, IO_WriteOnly);
	os.setEncoding(QTextStream::UnicodeUTF8);

	KLB::LabelManager * t = new KLB::LabelManager();
	t->addKeywords(name,lt);
	
	kdDebug()<<" added keywords"<<keywords << " to "<<name<<endl;
        
	os << "<html><head><title>K LABEL BROWSER</title></head><body>";
	os << "<br><br><br> <center> added key words  " << keywords << " to "<< name;
	os << "<br><br>Go to <A HREF=klb:/><b> KLB HOME</b> </A> </center>";
	os<< "</body></html>"<<endl;
    data(output);
	kdDebug()<<"executed data(output)"<<endl;
    finished();
	return;

}

void KlbProtocol::get( const KURL& url)
{
	int labelSelected = LABEL_SELECTED;

	    mimeType("text/html");
        kdDebug() << "GET " << url.url() << endl;

	kdDebug()<<"url path"<< url.path() <<"url.has path" <<url.hasPath() <<endl ;
	kdDebug()<<"url file "<< url.fileName() <<endl ;
	kdDebug()<<"url  "<< url.url()<<endl ;
	kdDebug()<<"pretty url  "<< url.prettyURL()<<endl ;
	kdDebug()<<"html url  "<< url.htmlURL()<<endl ;
	kdDebug()<<"query  "<< url.query()<<endl ;

//	if(!url.hasPath())

	if(url.path().compare("/")==0)	
	{
		labelSelected = NO_LABEL_SELECTED;
		showMain(url);
	}
	if(url.path().compare("/view")==0)
	{
		if(url.query()=="?" || url.query()==0)
		{
			showMain(url);
		}	
		else
		{
			labelCombination(url);
		}
	}
	else if(url.path().compare("/apply")==0)
	{
		if(url.query()=="?" || url.query()==0)
		{
			applyForm(url);
		}
		else
		{
			applyLabel(url);
		}
		//showMain();
	}
	else if(url.path().compare("/create")==0)
	{
		if(url.query()=="?" || url.query()==0)
		{
			createForm(url);
		}
		else
		{	
			createLabel(url);
		}
		//showMain();
	}
	else if(url.path().compare("/delete")==0)
	{
		deleteLabel(url);
	}
	else if(url.path().compare("/rename")==0)
	{
		renameLabel(url);
	}
	else if(url.path().compare("/remove")==0)
	{
		removeLabel(url);
	}
	else if(url.path().compare("/addkeywords")==0)
	{
		addKeywords(url);
	}
}

void KlbProtocol::mimetype( const KURL& )
{
    mimeType("text/html");
    finished();
}

extern "C"
{
    int KDE_EXPORT kdemain( int argc, char **argv ) {

        KInstance instance("kio_klb");

        if (argc != 4)
        {
            fprintf(stderr, "Usage: kio_klb protocol domain-socket1 domain-socket2\n");
            exit(-1);
        }

        KlbProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();

        return 0;
    }
}

