/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "klipper.h"

#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>

#include <KAction>
#include <KDebug>

#include "klipperwidget.h"

KlipperApplet::KlipperApplet(QObject *parent, const QVariantList &args)
    : Plasma::PopupApplet(parent, args), m_popup(0), m_klipWidget(0)
{
}

KlipperApplet::~KlipperApplet()
{
    delete m_popup;
}


void KlipperApplet::init()
{
    setPopupIcon("klipper");
    setGlobalShortcut(KShortcut(Qt::CTRL + Qt::ALT + Qt::Key_C));
}

QGraphicsWidget* KlipperApplet::graphicsWidget()
{
    if (!m_popup)
        createPopupWidget();

    return m_popup;
}

void KlipperApplet::createPopupWidget()
{
    /*
     * Create KlipperWidget
     */
    // try to load data engine
    Plasma::DataEngine* engine = dataEngine("klipper");
    m_klipWidget = new KlipperWidget;
    m_klipWidget->setClipboardEngine(engine);
    connect(m_klipWidget, SIGNAL(currentEntryChanged()), SLOT(hidePopup()));

    /*
     * Create popup widget
     */
    m_popup = new QGraphicsWidget(this);
    // m_klipWidget will be deleted by the proxy
    QGraphicsProxyWidget* proxy = new QGraphicsProxyWidget(m_popup);
    proxy->setWidget(m_klipWidget);

    QGraphicsLinearLayout* layout = new QGraphicsLinearLayout(m_popup);
    layout->addItem(proxy);

    m_popup->setMinimumSize(300,500);
}

/*
 * NOTE: Due to strange focus behavior I need to reimplement these two methods to get it working.
 * What happens:
 *   - If I leave setFocus() only in popupEvent() then widget doesn't receive focus when popup is
 *     triggered by clicking the mouse button on icon in panel, but does receive it if activated by
 *     shorctut
 *   - If I leave setFocus() only in focusInEvent() then things get opposite: widget receives focus
 *     if activated by clicking on panel icon, but it doesn't if popup is triggered by shortcut
 *
 * Let these both functions be here until I'll have a chance to figure out what happens
 */
void KlipperApplet::popupEvent(bool shown)
{
    // need to explicitly set focus to the widget inside popup,
    // or it will be unfocused
    if (shown)
        m_klipWidget->setFocus();
}
void KlipperApplet::focusInEvent(QFocusEvent*)
{
    m_klipWidget->setFocus();
}


void KlipperApplet::constraintsEvent(Plasma::Constraints constraints)
{
    Plasma::PopupApplet::constraintsEvent(constraints);
}

void KlipperApplet::onShowPopupActionTriggered()
{
    showPopup();
    /* not used anymore, simply show popup
     * keep for some time if i change my mind
    if (!m_klipWidget)
        m_klipWidget = createKlipperWidget();

    m_klipWidget->show();
    m_klipWidget->raise();
    */
}

#include "klipper.moc"
