/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "klipperengine.h"

#include <KDebug>
#include <KStandardDirs>

#include "klipperservice.h"
#include "klipboard.h"

static const char historyFileName[] = "klipper-history";

KlipperEngine::KlipperEngine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent, args)
{
    Q_UNUSED(args)
}


KlipperEngine::~KlipperEngine()
{
}

void KlipperEngine::init()
{
    kDebug() << KGlobal::mainComponent().componentName();
    // TODO config
    Klipboard::self()->setHistorySize(10);

    QString historyFile = KStandardDirs::locateLocal("data", QString("plasma_engine_klipper/%1").arg(historyFileName), true /* createDir */);
    Klipboard::self()->setHistoryFile(historyFile);

    // pick up existing data
    int count = Klipboard::self()->entryCount();
    for (int i=0; i<count; ++i)
        onClipEntryAdded(Klipboard::self()->entry(i).uuid);

    connect(Klipboard::self(),
            SIGNAL(entryAdded(const QString&)),
            SLOT(onClipEntryAdded(const QString&)));

    connect(Klipboard::self(),
            SIGNAL(entryRemoved(const QString&)),
            SLOT(onClipEntryRemoved(const QString&)));

    connect(Klipboard::self(),
            SIGNAL(entryRemoved(const QString&)),
            SLOT(onClipEntryRemoved(const QString&)));

    connect(Klipboard::self(),
            SIGNAL(currentEntryChanged(const QString&)),
            SLOT(onClipCurrentEntryChanged(const QString&)));
}

Plasma::Service* KlipperEngine::serviceForSource(const QString& source)
{
    Q_UNUSED(source)
    return new KlipperService(this);
}

void KlipperEngine::onClipEntryAdded(const QString& uuid)
{
    ClipboardEntry entry = Klipboard::self()->entry(uuid);
    if (!entry.isValid())
    {
        kDebug() << "refusing to add an invalid clipboard entry to engine, uuid passed:" << uuid;
        return;
    }

    kDebug() << uuid << entry.data << entry.isText << entry.isSelection;

    QString sourceName = uuid;
    Plasma::DataEngine::Data data;
    data["Data"] = entry.data;
    data["IsText"] = entry.isText;
    data["IsSelection"] = entry.isSelection;

    setData(sourceName, data);
}

void KlipperEngine::onClipEntryRemoved(const QString& uuid)
{
    removeSource(uuid);
}

void KlipperEngine::onClipCurrentEntryChanged(const QString& uuid)
{
    setData("currentEntryUuid", uuid);
}

#include "klipperengine.moc"
