/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "mergermodel.h"

#include <QtDBus/QDBusConnection>
#include <Nepomuk/Resource>

#include <KDebug>

Nepomuk::MergerModel::MergerModel(QObject* parent): QAbstractListModel(parent)
{

}

Nepomuk::MergerModel::~MergerModel()
{

}


QVariant Nepomuk::MergerModel::data(const QModelIndex& index, int role) const
{
    if( !index.isValid() )
        return QVariant();

    if( index.row() >= m_list.size() )
        return QVariant();

    if( role == Qt::DisplayRole ) {
        return m_list.at( index.row() ).label;
    }

    return QVariant();
}

int Nepomuk::MergerModel::rowCount(const QModelIndex& ) const
{
    return m_list.size();
}


bool Nepomuk::MergerModel::removeRows(int row, int count, const QModelIndex& parent)
{
    if( row + count >= m_list.size() || row >= m_list.size() )
        return false;

    beginRemoveRows( parent, row, row + count - 1);

    for( int i=row; i<count; i++ )
        m_list.removeAt( i );

    endRemoveRows();
    return true;
}


void Nepomuk::MergerModel::multipleMerge(const QString& uri, const QString& prop)
{
    kDebug() << uri << " " << prop;
    MergerRecord mr;
    mr.uri = QUrl( uri );
    mr.prop = QUrl( prop );
    mr.label = Nepomuk::Resource( uri ).genericLabel();

    //FIXME: Use better signals
    emit layoutAboutToBeChanged();
    m_list.append( mr );
    kDebug() << m_list.size();
    emit layoutChanged();
}


void Nepomuk::MergerModel::clearAll()
{
    emit layoutAboutToBeChanged();
    m_list.clear();
    emit layoutChanged();
}

Nepomuk::MergerRecord Nepomuk::MergerModel::record(int row)
{
    return m_list.at( row );
}

Nepomuk::MergerRecord Nepomuk::MergerModel::record(const QModelIndex& index)
{
    if( index.isValid() )
        return record( index.row() );
    else
        return MergerRecord();
}

void Nepomuk::MergerModel::remove(const QModelIndex& index)
{
    if( index.isValid() )
        m_list.removeAt( index.row() );
}
