/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "resolutiondialog.h"

#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Types/Property>
#include <Nepomuk/Types/Class>
#include <Nepomuk/Types/Literal>

#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>
#include <Soprano/NodeIterator>

#include <QtGui/QBoxLayout>
#include <QtGui/QGroupBox>

#include <KDebug>

Nepomuk::ResolutionDialog::ResolutionDialog(const QUrl& uri, const QUrl& prop, QWidget* parent, Qt::WFlags flags)
    : KDialog(parent, flags)
{
    m_resUri = uri;
    m_propUri = prop;

    QString query = QString::fromLatin1("select ?o where { %1 %2 ?o. }")
                    .arg( Soprano::Node::resourceToN3( m_resUri ),
                          Soprano::Node::resourceToN3( m_propUri ) );
    Soprano::Model * model = ResourceManager::instance()->mainModel();
    Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageSparql );
    m_objectNodes = it.iterateBindings("o").allNodes();
    
    //
    // Build the GUI
    //
    //TODO: Use something better in the future. Maybe what Oszkar is doing.
    QGroupBox * outerBox = new QGroupBox("Resolution Dialog");
    setMainWidget( outerBox );
    
    QVBoxLayout * layout = new QVBoxLayout( this );
    outerBox->setLayout( layout );

    QLabel * resLabel = new QLabel( m_resUri.toString(), this );
    QLabel * propLabel = new QLabel( m_propUri.toString(), this );
    QGroupBox * box = new QGroupBox( "Options", this );

    layout->addWidget( resLabel );
    layout->addWidget( propLabel );
    layout->addWidget( box );
    
    QVBoxLayout * boxLayout = new QVBoxLayout( box );
    foreach( const Soprano::Node & node, m_objectNodes ) {
        QString label;
        if( node.isResource() ) {
            label = Resource( node.uri() ).genericLabel();
            if( label.isEmpty() )
                label = node.uri().toString();
        }
        else if( node.isLiteral() )
            label = node.toN3();

        kDebug() << node.uri()<<  "  Label :  " << label;
        QCheckBox * checkBox = new QCheckBox( label, this );
        checkBox->setChecked( true );
        m_checkBoxes << checkBox;
        boxLayout->addWidget( checkBox );
    }
    box->setLayout( boxLayout );
}

void Nepomuk::ResolutionDialog::accept()
{
    Soprano::Statement st( m_resUri, m_propUri, Soprano::Node(), Soprano::Node() );
    Soprano::Model * model = ResourceManager::instance()->mainModel();

    for( int i=0; i<m_checkBoxes.size(); i++ ) {
        QCheckBox * checkBox = m_checkBoxes[i];
        if( checkBox->checkState() == Qt::Unchecked ) {
            // Remove it from the model
            st.setObject( m_objectNodes[i] );
            model->removeAllStatements( st );
        }
    }
    QDialog::accept();
}
