/* This file is part of the Nepomuk Project
   Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <QMenu>
#include <QString>
#include <QtDBus>
#include <QDebug>
#include <QMap>

#include <KAction>
#include <KGenericFactory>
#include <KFileItemList>
#include <KDialog>
#include <KMessageBox>

#include <Soprano/Vocabulary/XMLSchema>
#include <Soprano/Node>
#include <Soprano/QueryResultIterator>
#include <Soprano/Model>
#include <Nepomuk/ResourceManager>
#include <Nepomuk/Resource>
#include <Nepomuk/Variant>

#include "konqnepomukplugin.h"
#include "nco.h"
#include "contactswidget.h"

class KonqNepomukPluginFactory : public KGenericFactory<KonqNepomukPlugin, KonqPopupMenu>
{
    public:
        KonqNepomukPluginFactory(const char* instanceName = 0): KGenericFactory<KonqNepomukPlugin, KonqPopupMenu>(instanceName)
        {
        }

        ~KonqNepomukPluginFactory()
        {
        }
};

K_EXPORT_COMPONENT_FACTORY (libkonqnepomukplugin, KonqNepomukPluginFactory ("konqnepomukplugin"))

KonqNepomukPlugin::KonqNepomukPlugin(KonqPopupMenu* popupMenu, const QStringList& list): KonqPopupMenuPlugin(popupMenu)
{
    Q_UNUSED(list);
}

KonqNepomukPlugin::~KonqNepomukPlugin()
{
}

void KonqNepomukPlugin::setup(KActionCollection* actionCollection, const KonqPopupMenuInformation& popupMenuInfo, QMenu* menu)
{
    Nepomuk::ResourceManager::instance()->init();
    if (Nepomuk::ResourceManager::instance()->initialized()) {
        KFileItemList itemList = popupMenuInfo.items();
        foreach (const KFileItem& item, itemList) {
            if (!item.isDir()) {
                Nepomuk::Resource r(item.localPath());
                m_files.append(r.uri());
            }
        }
        
        KAction* shareAction = new KAction(KIcon("nepomuk"), i18n("Share with friends"), actionCollection);
        connect(shareAction, SIGNAL(triggered(bool)), this, SLOT(showSharingDialog()));
        menu->addAction(shareAction);
    }
}

void KonqNepomukPlugin::showSharingDialog()
{
    QMap<QString, QString> contacts;
    QString query = QString("SELECT ?fn ?c WHERE { ?c a %1 . ?c %2 ?a . ?a %3 \"Jabber\"^^%4 . ?a %5 ?id . ?c %6 ?fn . }")
        .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::NCO::PersonContact()))
        .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::NCO::hasIMAccount()))
        .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::NCO::imAccountType()))
        .arg(Soprano::Node::resourceToN3(Soprano::Vocabulary::XMLSchema::string()))
        .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::NCO::imID()))
        .arg(Soprano::Node::resourceToN3(Nepomuk::Vocabulary::NCO::fullname()));
    qDebug() << query;
    Soprano::QueryResultIterator it = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery(query, Soprano::Query::QueryLanguageSparql);
    while (it.next()) {
        Soprano::Node contact = it.binding("c");
        Soprano::Node name = it.binding("fn");
        contacts[name.toString()] = contact.toString();
    }
    
    if (contacts.size() > 0) {
        KDialog* dlg = new KDialog();
        dlg->setCaption("Share with friends");
        dlg->setButtons(KDialog::Ok | KDialog::Cancel);
        ContactsWidget* cw = new ContactsWidget(contacts.keys(), dlg);
        dlg->setMainWidget(cw);
        connect(dlg, SIGNAL(okClicked()), this, SLOT(share()));
        dlg->show();
    }
    else {
        KMessageBox::sorry(0, i18n("You don't have any friend to share with."), "Share with friends");
    }
}

void KonqNepomukPlugin::share()
{
    
}
