/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "metadatasharing.h"
#include "nfo.h"
#include <KDebug>

#include <nepomuk/identificationset.h>
#include <Nepomuk/Resource>
#include <Nepomuk/ResourceManager>

#include <Soprano/Model>
#include <Soprano/QueryResultIterator>
#include <Soprano/Serializer>
#include <Soprano/Parser>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/PluginManager>
#include <Soprano/StatementIterator>
#include <Soprano/Util/SimpleStatementIterator>

Nepomuk::MetadataSharing::MetadataSharing( QObject* parent, const QList< QVariant >& )
	: Service(parent)
{
    kDebug();
}

Nepomuk::MetadataSharing::~MetadataSharing()
{
}


void Nepomuk::MetadataSharing::metadataUriReceived( const QString & fromId, const QString & uri )
{
    QString uuid = uri.mid( 8 );
    QUrl newUri( "nepomuk:/" + fromId + "/res/" + uuid );

    Resource res( newUri );
    res.addType( Nepomuk::Vocabulary::NFO::RemoteDataObject() );

    //TODO: Get the contact with id fromId, and attach it to the new resource
    //QString query
    //Soprano::Model * model = ResourceManager::instance()->mainModel();

    emit requestMetadata( fromId, uri );
}


void Nepomuk::MetadataSharing::metadataReceived( const QString& fromId, const QString& uri, const QString& serializedMetadata )
{
    QString uuid = uri.mid( 8 );
    QUrl resUri( "nepomuk:/" + fromId + "/res/" + uuid );
    QUrl origUri( uri );

    Soprano::Model * model = ResourceManager::instance()->mainModel();
    const Soprano::Parser * parser = Soprano::PluginManager::instance()->discoverParserForSerialization( Soprano::SerializationNQuads );

    QString metadata( serializedMetadata );
    QTextStream stream( &metadata );
    Soprano::StatementIterator iter = parser->parseStream( stream, QUrl(), Soprano::SerializationNQuads );
    while( iter.next() ) {
        Soprano::Statement st = iter.current();
        // Change the subject
        if( st.subject().uri() == origUri ) {
            st.setSubject( resUri );
        }

        //FIXME: We should probably create a new resource NFO:RemoteDataObject resource for each object
        // which is a resource.
        model->addStatement( st );
    }
}


void Nepomuk::MetadataSharing::metadataRequest( const QString & fromId, const QString & uri )
{
    //TODO: Check for permissions?

    // Get all the properties + identifyable properties of the all objects which are resources
    QList<Soprano::Statement> statements;
    QList<QUrl> objects;

    QString query = QString::fromLatin1("select ?p ?o where { %1 ?p ?o . }")
                    .arg( Soprano::Node::resourceToN3( QUrl(uri) ) );
    Soprano::Model * model = ResourceManager::instance()->mainModel();
    Soprano::QueryResultIterator it = model->executeQuery( query, Soprano::Query::QueryLanguageSparql );

    while( it.next() ) {
        Soprano::Node o = it["o"];
        statements << Soprano::Statement( QUrl(uri), it["p"], o, Soprano::Node() );
        if( o.isResource() && o.uri().scheme() == "nepomuk" )
            objects << o.uri();
    }

    statements << Sync::IdentificationSet::fromResourceList( objects ).toList();

    if( statements.empty() )
        return;
    
    //
    // Serialize the statements
    //
    //FIXME: Use an XML format!
    const Soprano::Serializer * serializer = Soprano::PluginManager::instance()->discoverSerializerForSerialization( Soprano::SerializationNQuads );

    QString output;
    QTextStream out( &output );
    Soprano::Util::SimpleStatementIterator iter( statements );
    serializer->serialize( iter, out, Soprano::SerializationNQuads );

    emit publishMetadata( fromId, uri, output );
}

#include <kpluginfactory.h>
#include <kpluginloader.h>

NEPOMUK_EXPORT_SERVICE( Nepomuk::MetadataSharing, "nepomukmetadatasharing" )

#include "metadatasharing.moc"