/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "taskresourceeditplugin.h"
#include "nfo.h"

#include "../src/propertyeditwidget.h"
#include "../src/nepomukdataaccess.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPalette>
#include <QColor>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QSet>
#include <QUrl>
#include <QLabel>
#include <KComboBox>
#include <QMap>

#include <Nepomuk/Types/Property>
#include <Nepomuk/Variant>
#include <Nepomuk/Resource>
#include <Soprano/Vocabulary/RDFS>

#include <KPluginFactory>
#include <KPluginLoader>
#include <KDebug>
#include <KLocalizedString>

#include <Soprano/Vocabulary/NAO>
#include "tmo.h"

K_PLUGIN_FACTORY(TaskResourceEditPluginFactory, registerPlugin<Nepomuk::TaskResourceEditPlugin>();)
K_EXPORT_PLUGIN(TaskResourceEditPluginFactory("nepomuk_taskresourceeditplugin"))

class Nepomuk::TaskResourceEditPlugin::Private
{
public:
    Private() {
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Archived().toString(),
                            i18n("Archived"));
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Completed().toString(),
                            i18n("Completed"));
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Deleted().toString(),
                            i18n("Deleted"));
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Finalized().toString(),
                            i18n("Finalized"));
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_New().toString(),
                            i18n("New"));
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Running().toString(),
                            i18n("Running"));
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Suspended().toString(),
                            i18n("Suspended"));
        stateStrings.insert(Nepomuk::Vocabulary::TMO::TMO_Instance_TaskState_Terminated().toString(),
                            i18n("Terminated"));
    }

    KComboBox *stateComboBox;
    QMap<QString,QString> stateStrings;
};

Nepomuk::TaskResourceEditPlugin::TaskResourceEditPlugin(QWidget *parent, const QList<QVariant>& ) :
        AbstractResourceEditPlugin(parent),
        d( new Private() )
{
}

Nepomuk::TaskResourceEditPlugin::~TaskResourceEditPlugin()
{
    delete d;
}

void Nepomuk::TaskResourceEditPlugin::initGui()
{
    //<trueg> tasks should contain label, description, supertask, and task state
    //<trueg> and maybe due date
    if (layout()) {
        layout()->deleteLater();
    }

    //add main layout
    QVBoxLayout *mainLayout = new QVBoxLayout();
    mainLayout->setContentsMargins(0, 0, 0, 0);

    //add property editor widgets (and their layout)
    QFormLayout *formLayout = new QFormLayout;
    formLayout->setContentsMargins(1, 1, 1, 1);
    mainLayout->addLayout(formLayout);

    //QSet<QUrl> properties = resource().properties().keys().toSet();//build list of properties
    QList<QUrl> properties;
    properties << Soprano::Vocabulary::NAO::prefLabel()
               << Soprano::Vocabulary::RDFS::comment()
               << Nepomuk::Vocabulary::TMO::superTask()
               << Nepomuk::Vocabulary::TMO::dueDate();
    //QList<QUrl> mandatoryProperties = NepomukDataAccess::mandatoryPropertiesForType(resource().resourceType());
    //properties.unite(mandatoryProperties.toSet());

    foreach(QUrl propertyUrl, properties) { //create editor widgets
        Nepomuk::Types::Property property(propertyUrl);
        PropertyEditWidget *propertyWidget = new PropertyEditWidget(this,
                                                                    property,
                                                                    resource().property(propertyUrl));

        QLabel *label = new QLabel();
        label->setText("<font color='#808080'>" + property.label() + "</font>:");//grey text
        label->setToolTip(property.uri().toString());

        formLayout->addRow(label, propertyWidget);
        addPropertyEditWidget(propertyWidget);
    }

    //state combobox
    d->stateComboBox = new KComboBox();
    d->stateComboBox->addItems(d->stateStrings.values());
    Nepomuk::Resource taskState = resource().property(Nepomuk::Vocabulary::TMO::taskState()).toResource();
    d->stateComboBox->setCurrentItem(d->stateStrings.value(taskState.uri()));//workaround for inexistent TaskState instance label
    d->stateComboBox->setEnabled(false);

    QLabel *label = new QLabel();
    label->setText("<font color='#808080'>" + QString("Task state") + "</font>:");//grey text
    label->setToolTip(Nepomuk::Vocabulary::TMO::taskState().toString());

    formLayout->addRow(label, d->stateComboBox);


    //set layout and focus policy
    setLayout(mainLayout);
}


void Nepomuk::TaskResourceEditPlugin::setEditingEnabled(bool enable)
{
    AbstractResourceEditPlugin::setEditingEnabled(enable);

    d->stateComboBox->setEnabled(enable);
}

void Nepomuk::TaskResourceEditPlugin::revertChanges()
{
    AbstractResourceEditPlugin::revertChanges();

    Nepomuk::Resource taskState = resource().property(Nepomuk::Vocabulary::TMO::taskState()).toResource();
    d->stateComboBox->setCurrentItem(d->stateStrings.value(taskState.uri()));//workaround for inexistent TaskState instance label
}


void Nepomuk::TaskResourceEditPlugin::saveChanges()
{
    AbstractResourceEditPlugin::saveChanges();

    QStringList currentKeys = d->stateStrings.keys(d->stateComboBox->currentText());
    if(!currentKeys.isEmpty()) {
        QString uriString = currentKeys.first();
        if(!uriString.isEmpty()) {
            resource().setProperty(Nepomuk::Vocabulary::TMO::taskState(), QUrl(uriString));
        }
    }
}
