/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QUERYBUILDERWIDGET_H
#define QUERYBUILDERWIDGET_H

#include "searchwidget.h"

#include <QWidget>

#include "nepomuksearchgui_export.h"

class QShowEvent;


namespace Nepomuk {
    namespace Query {
        class Term;
        class Query;

        /**
         * A widget for building Nepomuk queries. It uses ReseourceSearchWidget to configure the query.
         * It provides the current query to users of this component.
         */

        class NEPOMUKSEARCHGUI_EXPORT QueryBuilderWidget : public SearchWidget
        {
            Q_OBJECT;

        public:
            /**
             * Creates a new QueryBuilderWidget.
             */
            QueryBuilderWidget(QWidget *parent = 0);
            ~QueryBuilderWidget();

            /**
             * @return the current query
             */
            Nepomuk::Query::Query currentQuery() const;

            /**
             * Sets the query of the LineEdit component to @p term
             */
            //void setLineEditQuery(const Nepomuk::Query::Query &query);

            /**
             * @returns the query of the LineEdit component
             */
            //Nepomuk::Query::Query lineEditQuery() const;


            /**
             * Sets the query of the Facets widget @p term
             */
            //void setFacetsWidgetQuery(const Nepomuk::Query::Query &query);

            /**
             * @returns the query of the Facets widget
             */
            //Nepomuk::Query::Query facetsWidgetQuery() const;

        Q_SIGNALS:
            /**
             * Acivated when the query is changed
             */
            void queryChanged(const Nepomuk::Query::Query &query);

        protected:
            class QueryBuilderWidgetPrivate;

        private Q_SLOTS:
            void emitQueryChanged();

        private:
            Q_DECLARE_PRIVATE(QueryBuilderWidget)
        };
    }
}

#endif // QUERYBUILDERWIDGET_H
