/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "resourceeditwidgetwrapper.h"

#include "resourceeditwidget.h"

#include "abstractresourceeditplugin.h"
#include "resourceeditpluginfactory.h"

#include <QWidget>
#include <QVariant>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QAction>

#include <KDebug>
#include <KPushButton>

#include <Nepomuk/Resource>


ResourceEditWidgetWrapper::ResourceEditWidgetWrapper(QWidget *parent)
    : QWidget(parent), m_resourceEditWidget(0), m_btnEnableDisable(0), m_btnSave(0), m_btnRevert(0), m_editingEnabled(false)
{
    m_btnEnableDisable = new KPushButton(i18n("Enable"), this);
    connect(m_btnEnableDisable, SIGNAL(clicked()), this, SLOT(btnEnableDisableClicked()));

    m_btnSave = new KPushButton(i18n("Save"), this);    
    m_btnRevert = new KPushButton(i18n("Revert"), this);    

    QHBoxLayout *hLayout = new QHBoxLayout();
    hLayout->setContentsMargins(1, 1, 1, 1);
    hLayout->addStretch(7);
    hLayout->addWidget(m_btnEnableDisable, 1);
    hLayout->addWidget(m_btnSave, 1);
    hLayout->addWidget(m_btnRevert, 1);

    //layout and config
    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addLayout(hLayout);

    if(m_resourceEditWidget) {
        mainLayout->addWidget(m_resourceEditWidget);
    }

    setFocusPolicy(Qt::WheelFocus);
}

void ResourceEditWidgetWrapper::setResourceEditWidget(Nepomuk::ResourceEditWidget *widget)
{
    disconnect(m_btnSave, SIGNAL(clicked()), m_resourceEditWidget, SLOT(saveChanges()));
    disconnect(m_btnRevert, SIGNAL(clicked()), m_resourceEditWidget, SLOT(revertChanges()));

    if(m_resourceEditWidget) {
        layout()->removeWidget(m_resourceEditWidget);
    }

    m_resourceEditWidget = widget;   
    layout()->addWidget(m_resourceEditWidget);

    connect(m_btnSave, SIGNAL(clicked()), m_resourceEditWidget, SLOT(saveChanges()));
    connect(m_btnRevert, SIGNAL(clicked()), m_resourceEditWidget, SLOT(revertChanges()));
}

Nepomuk::ResourceEditWidget *ResourceEditWidgetWrapper::resourceEditWidget() const
{
    return m_resourceEditWidget;
}

void ResourceEditWidgetWrapper::btnEnableDisableClicked()
{
    if (m_editingEnabled) {
        m_resourceEditWidget->setEditingEnabled(false);
        m_btnEnableDisable->setText(i18n("Enable"));
        m_editingEnabled = false;
    } else {//enable editing
        m_resourceEditWidget->setEditingEnabled(true);
        m_btnEnableDisable->setText(i18n("Disable"));
        m_editingEnabled = true;
    }

    m_resourceEditWidget->updateGeometry();
    //FIXME: when anoter item closely below is open, this overlaps it
    //kDebug() << "*** Edit/Save: size:" << this->size();
}

void ResourceEditWidgetWrapper::focusInEvent(QFocusEvent *event)
{
    QWidget::focusInEvent(event);

    /*m_resourceEditWidget->setEditingEnabled(true);
    kDebug() << "*** REW Enabled";
    */
}

void ResourceEditWidgetWrapper::focusOutEvent(QFocusEvent *event)
{
    QWidget::focusOutEvent(event);

    /*m_resourceEditWidget->setEditingEnabled(false);
    kDebug() << "*** REW Disabled";
    */
}
