/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "testwidget.h"
#include "../src/facetwidget.h"

#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>
#include <KService>
#include <KServiceTypeTrader>
#include <KDebug>

#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/GroupTerm>
#include <Nepomuk/Query/Term>
#include <Nepomuk/Query/AndTerm>
#include <Nepomuk/Query/OrTerm>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/Query/LiteralTerm>
#include <Nepomuk/Query/ResourceTerm>
#include <Nepomuk/Query/ResourceTypeTerm>
#include <Nepomuk/Tag>
#include <Nepomuk/Types/Class>

#include <QUrl>
#include <QtTest>

void testFacetsWidget()
{
    Nepomuk::Utils::FacetWidget *widget = new Nepomuk::Utils::FacetWidget();
    widget->show();
    Nepomuk::Query::Query query;
    Nepomuk::Query::AndTerm andTerm;

    Nepomuk::Query::AndTerm tagTerm;

#warning aoszkar: This is just a local test on my machine
    tagTerm.addSubTerm(Nepomuk::Types::Property(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#hasTag")) ==
                       Nepomuk::Query::ResourceTerm(QUrl("nepomuk:/res/7e1e98ce-6b7f-4d4b-8cbd-1e312596e8ee")));
    tagTerm.addSubTerm(Nepomuk::Types::Property(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#hasTag")) ==
                       Nepomuk::Query::ResourceTerm(QUrl("nepomuk:/res/4b702846-62ef-4bc0-b6e5-049f3211b3b7")));

    andTerm.addSubTerm(Nepomuk::Query::ResourceTypeTerm(Nepomuk::Types::Class(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact"))));
    andTerm.addSubTerm(tagTerm);
    query.setTerm(andTerm);

    widget->extractFacetsFromTerm(query.term());
    Nepomuk::Query::Query before(widget->queryTerm());
    kDebug() << "*** before *** " << before;
    //check if we set the query constructed from the Facet widget, we get the same constructed query again
    widget->extractFacetsFromTerm(widget->queryTerm());
    Nepomuk::Query::Query after(widget->queryTerm());
    kDebug() << "*** after *** " << after;
    kDebug() << "*** COMPARING before == after:" << (before == after);
}


int main(int argc, char* argv[])
{
    KAboutData aboutData("nepomuksearchguitest", 0,
                         ki18n("Nepomuk Search GUI Test"),
                         "0.1",
                         ki18n("Nepomuk Search GUI Test"),
                         KAboutData::License_GPL,
                         ki18n("(c) 2010, Oszkar Ambrus"),
                         KLocalizedString(),
                         "http://nepomuk.kde.org");
    aboutData.addAuthor(ki18n("Oszkar Ambrus"), ki18n("Maintainer"), "aoszkar@gmail.com");
    aboutData.setProgramIconName("nepomuk");

    KCmdLineArgs::init(argc, argv, &aboutData);
    KApplication app;

    KService::Ptr service = KService::serviceByStorageId("kmail");

    testFacetsWidget();

    //(new TestWidget())->show();;

//    (new Nepomuk::QueryBuilderWidget())->show();

    return app.exec();
}

