/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "propertyeditwidgettest.h"
#include "../src/propertyeditwidget.h"

#include <QUrl>
#include <QVariant>

#include <Nepomuk/Variant>
#include <Nepomuk/Resource>
#include <Nepomuk/Types/Property>

#include <KGlobal>

PropertyEditWidgetTest::PropertyEditWidgetTest(QObject *parent)
    : QObject(parent), m_pew(new Nepomuk::PropertyEditWidget)
{
    KGlobal::locale();//initialize
}

PropertyEditWidgetTest::~PropertyEditWidgetTest()
{
    delete m_pew;
}

void PropertyEditWidgetTest::performTestSteps(const QUrl &url, const Nepomuk::Variant &variant)
{
    Nepomuk::Types::Property property(url);
    m_pew->setNepomukProperty(property);
    QCOMPARE(property, m_pew->nepomukProperty());

    m_pew->setValue(variant);
    m_pew->setEditingEnabled(false);
    m_pew->setEditingEnabled(true);
    m_pew->setEditingEnabled(false);
    QCOMPARE(variant, m_pew->value());
}

void PropertyEditWidgetTest::testDate()
{
    //birthDate
    performTestSteps(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#birthDate"), QDate::currentDate());
}

void PropertyEditWidgetTest::testDateTime()
{
    performTestSteps(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#lastModified"), QDateTime::currentDateTime());
}

void PropertyEditWidgetTest::testInt()
{
    performTestSteps(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#numericRating"), 7);
}

void PropertyEditWidgetTest::testString()
{
     performTestSteps(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#fullname"), "Robert Cleaver Chapman");
}

void PropertyEditWidgetTest::testDouble()
{
    performTestSteps(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#rate"), 5.42);
}

void PropertyEditWidgetTest::testBool()
{
    performTestSteps(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#isPasswordProtected"), true);
}

void PropertyEditWidgetTest::testResource()
{
    QSKIP("Uses local data", SkipSingle);
#warning aoszkar: This is just a local test on my machine
    performTestSteps(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#fileOwner"), Nepomuk::Resource(QUrl("nepomuk:/res:/dummycontact1")));
}

void PropertyEditWidgetTest::testStringMultiple()
{
    QStringList l;
    l << "String 1 " << "String 2" << "String 3";    
    performTestSteps(QUrl("http://www.w3.org/2000/01/rdf-schema#label"), l);
}

void PropertyEditWidgetTest::testResourceMultiple()
{
    QSKIP("Uses local data", SkipSingle);
#warning aoszkar: This is just a local test on my machine
    QList<Nepomuk::Resource> l;
    l << Nepomuk::Resource(QUrl("nepomuk:/res/7e1e98ce-6b7f-4d4b-8cbd-1e312596e8ee"))
      << Nepomuk::Resource(QUrl("nepomuk:/res/b3303cb3-8537-46d0-943b-cf898cc93b3f"))
      << Nepomuk::Resource(QUrl("nepomuk:/res/cf1186e5-450d-4378-8bcd-ef9fdd4b1e8b"));
    performTestSteps(QUrl("http://www.semanticdesktop.org/ntologies/2007/08/15/nao#hasTag"), l);
}

QTEST_KDEMAIN(PropertyEditWidgetTest, GUI)
#include "propertyeditwidgettest.moc"
