/*
 * search_widget_testview.h
 *
 * Copyright (C) 2007 Adam Kidder <thekidder@gmail.com>
 */
#ifndef SEARCH_WIDGET_TESTVIEW_H
#define SEARCH_WIDGET_TESTVIEW_H

#include <QMap>
#include <QtGui/QWidget>

#include <KConfig>
#include <KDirModel>

#include "ui_search_widget_testview_base.h"

class QPainter;
class KUrl;

/**
 * This is the main view class for search_widget_test.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * @short Main view
 * @author Adam Kidder <thekidder@gmail.com>
 * @version 0.1
 */

class search_widget_testView : public QWidget, public Ui::search_widget_testview_base
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    search_widget_testView(QWidget *parent);

    /**
     * Destructor
     */
    virtual ~search_widget_testView();

public slots:
    void searchStringChanged(const QString& searchString);
    void advancedSearchChanged(const QString& advancedString);
    void activateSearch();
    void saveSearch();

private:
    QString getQueryString();

    void loadSavedSearches();
    void saveSavedSearches();

    Ui::search_widget_testview_base ui_search_widget_testview_base;
    KDirModel* model;
    QString search;
    QString advanced;
    QString searchName;
    KConfig savedSearches;

private slots:
    void setName(const QString& name);
    void doSaveSearch();

signals:
    /**
     * Use this signal to change the content of the statusbar
     */
    void signalChangeStatusbar(const QString& text);

    /**
     * Use this signal to change the content of the caption
     */
    void signalChangeCaption(const QString& text);
};

#endif // search_widget_testVIEW_H
