/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "testdata.h"

#include "graphutility.h"

#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QUrl>

#include <Soprano/LiteralValue>
#include <Soprano/Model>
#include <Soprano/Statement>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/RDFS>

Soprano::Model* TestData::createExampleModel()
{
    Soprano::Model* result = SemClip::createModel();

    result->addStatement( Soprano::Statement( QUrl( "http://www.example.org/data/TobiasWolf" ),
                                              QUrl( "http://www.example.org/onto1/ResourceProperty" ),
                                              QUrl( "http://www.example.org/data/SomeResource" ) ) );

    result->addStatement( Soprano::Statement( QUrl( "http://www.example.org/data/TobiasWolf" ),
                                              QUrl( "http://www.example.org/onto2/LiteralProperty" ),
                                              Soprano::LiteralValue( "SomeLiteralValue" ) ) );

    return result;
}

QString TestData::exampleRdfXmlSerialization()
{
    return QString( "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
                    "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\""
                    "         xmlns:ex1=\"http://www.example.org/onto1/\""
                    "         xmlns:ex2=\"http://www.example.org/onto2/\">"
                    "    <rdf:Description rdf:about=\"http://www.example.org/data/TobiasWolf\">"
                    "        <ex2:LiteralProperty>SomeLiteralValue</ex2:LiteralProperty>"
                    "    </rdf:Description>"
                    "    <rdf:Description rdf:about=\"http://www.example.org/data/TobiasWolf\">"
                    "        <ex1:ResourceProperty rdf:resource=\"http://www.example.org/data/SomeResource\"/>"
                    "    </rdf:Description>"
                    "</rdf:RDF>" );
}

QSet< QUrl > TestData::exampleOntologies()
{
    QSet< QUrl > result;
    result.insert( QUrl( "http://www.example.org/onto1/") );
    result.insert( QUrl( "http://www.example.org/onto2/" ) );
    return result;
}

QSet< QUrl > TestData::exampleProperties()
{
    QSet< QUrl > result;
    result.insert( QUrl( "http://www.example.org/onto1/ResourceProperty") );
    result.insert( QUrl( "http://www.example.org/onto2/LiteralProperty" ) );
    return result;
}

QSet< QUrl > TestData::exampleResources()
{
    QSet< QUrl > result;
    result.insert( QUrl( "http://www.example.org/data/SomeResource") );
    return result;
}

Soprano::Model* TestData::createExampleOntology()
{
    Soprano::Model* result = SemClip::createModel();

    result->addStatement( Soprano::Statement( QUrl( "http://www.example.org/onto1/SomeClass" ),
                                              Soprano::Vocabulary::RDF::type(),
                                              Soprano::Vocabulary::RDFS::Class() ) );

    result->addStatement( Soprano::Statement( QUrl( "http://www.example.org/onto1/AnotherClass" ),
                                              Soprano::Vocabulary::RDF::type(),
                                              Soprano::Vocabulary::RDFS::Class() ) );

    result->addStatement( Soprano::Statement( QUrl( "http://www.example.org/onto1/ResourceProperty" ),
                                              Soprano::Vocabulary::RDF::type(),
                                              Soprano::Vocabulary::RDF::Property() ) );

    result->addStatement( Soprano::Statement( QUrl( "http://www.example.org/onto1/ResourceProperty" ),
                                              Soprano::Vocabulary::RDFS::domain(),
                                              QUrl( "http://www.example.org/onto1/SomeClass" ) ) );

    result->addStatement( Soprano::Statement( QUrl( "http://www.example.org/onto1/ResourceProperty" ),
                                              Soprano::Vocabulary::RDFS::range(),
                                              QUrl( "http://www.example.org/onto1/AnotherClass" ) ) );

    return result;
}

QString TestData::exampleInferenceQuery()
{
    return QString( "ask {"
                    " <http://www.example.org/data/TobiasWolf> <%1> <http://www.example.org/onto1/SomeClass> ."
                    " <http://www.example.org/data/SomeResource> <%1> <http://www.example.org/onto1/AnotherClass>"
                    "}" )
           .arg( Soprano::Vocabulary::RDF::type().toString() );
}

QUrl TestData::exampleOntologyUri()
{
    return QUrl( "http://www.example.org/onto1/" );
}

QString TestData::identifyTransformation()
{
    return QString( "construct { ?s ?p ?o } where { ?s ?p ?o }\n" );
}

QString TestData::nullTransformation()
{
    return QString( "construct { ?s ?p ?o } where { ?s ?p ?o . ?p ?s ?o }\n" );
}

QMap< QString, QUrl > TestData::examplePrefixMap()
{
    QMap< QString, QUrl > result;
    result.insert( QString( "one" ), QUrl( "http://www.example.org/onto1/") );
    result.insert( QString( "two" ), QUrl( "http://www.example.org/onto2/") );
    return result;
}

QStringList TestData::exampleMappingQueries()
{
    QStringList result;
    result.append( identifyTransformation() );
    return result;
}

QString TestData::exampleMappingFileName()
{
    return QString( "one2two.zip" );
}
