/* This file is part of the Nepomuk Project
 Copyright (c) 2009 Alessandro Sivieri <alessandro.sivieri@gmail.com>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License version 2 as published by the Free Software Foundation.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 Boston, MA 02110-1301, USA.
 */

#include <KApplication>
#include <KAction>
#include <KLocale>
#include <KActionCollection>
#include <KStandardAction>
#include <KMessageBox>
#include <KIO/NetAccess>
#include <KSaveFile>
#include <QTextStream>
#include <QStringList>
#include <Nepomuk/Types/Class>

#include "mainwindow.h"
#include <smartfiledialog.h>
#include <nfo.h>
#include <pimo.h>

using namespace Test1;

MainWindow::MainWindow(QWidget *parent) :
    KXmlGuiWindow(parent), fileName(QString())
{
    textArea = new KTextEdit();
    setCentralWidget(textArea);
    setupActions();
}

void MainWindow::openFile(const QString &inputFileName)
{
    if (inputFileName.length() > 0)
    {
        QString tmpFile;
        if (KIO::NetAccess::download(inputFileName, tmpFile, this))
        {
            QFile file(tmpFile);
            file.open(QIODevice::ReadOnly);
            textArea->setPlainText(QTextStream(&file).readAll());
            fileName = inputFileName;
            KIO::NetAccess::removeTempFile(tmpFile);
        }
        else
        {
            KMessageBox::error(this, KIO::NetAccess::lastErrorString());
        }
    }
}

void MainWindow::setupActions()
{
    KStandardAction::quit(kapp, SLOT(quit()), actionCollection());
    KStandardAction::open(this, SLOT(openFile()), actionCollection());
    KStandardAction::save(this, SLOT(saveFile()), actionCollection());
    KStandardAction::saveAs(this, SLOT(saveFileAs()), actionCollection());
    KStandardAction::openNew(this, SLOT(newFile()), actionCollection());
    setupGUI();
}

void MainWindow::newFile()
{
    fileName.clear();
    textArea->clear();
}

void MainWindow::openFile()
{
    Nepomuk::Types::Class cls(Nepomuk::Vocabulary::NFO::PlainTextDocument());
    Nepomuk::Types::Class pimocls(Nepomuk::Vocabulary::PIMO::Document());
    openFile(Nepomuk::SmartFileDialog::getOpenFileName(cls, this, pimocls));
}

void MainWindow::saveFile()
{
    if (!fileName.isEmpty())
    {
        saveFileAs(fileName);
    }
    else
    {
        saveFileAs();
    }
}

void MainWindow::saveFileAs()
{
    Nepomuk::Types::Class cls(Nepomuk::Vocabulary::NFO::PlainTextDocument());
    Nepomuk::Types::Class pimocls(Nepomuk::Vocabulary::PIMO::Document());
    Nepomuk::SmartFileDialog dlg(cls, Nepomuk::SmartFileDialog::Saving, this, pimocls);
    dlg.setContent(textArea->toPlainText());
    QStringList mimes;
    mimes << "text/plain";
    mimes << "application/xml";
    dlg.setMimeFilter(mimes, mimes[0]);
    if (dlg.exec() == KDialog::Accepted) {
        saveFileAs(dlg.selectedFile());
    }
    //saveFileAs(Nepomuk::SmartFileDialog::getSaveFileName(cls, this, pimocls, QList<
    //        Nepomuk::Annotation*> (), textArea->toPlainText()));
}

void MainWindow::saveFileAs(const QString &outputFileName)
{
    if (outputFileName.length() > 0)
    {
        KSaveFile file(outputFileName);
        file.open();
        QByteArray outputByteArray;
        outputByteArray.append(textArea->toPlainText().toUtf8());
        file.write(outputByteArray);
        file.finalize();
        file.close();
        fileName = outputFileName;
    }
}
