#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <assert.h>

#include <security/pam_appl.h>

static int conv_func (int n, const struct pam_message **msg, struct pam_response **resp, void *arg)
{
        struct pam_response *aresp;
        int i;

   assert (n > 0 && n < PAM_MAX_NUM_MSG);
   aresp = (struct pam_response*)malloc(sizeof(struct pam_response) * n);

        for (i = 0; i < n; ++i) {
      aresp[i].resp_retcode = 0;
      aresp[i].resp = NULL;
      switch (msg[i]->msg_style) {
      case PAM_PROMPT_ECHO_OFF:
         aresp[i].resp = getpass (msg[i]->msg);
         assert (aresp[i].resp != NULL);
         break;
      case PAM_PROMPT_ECHO_ON:
         aresp[i].resp = getpass (msg[i]->msg);
         assert (aresp[i].resp != NULL);
         break;
      case PAM_ERROR_MSG:
         fputs(msg[i]->msg, stderr);
         if (strlen(msg[i]->msg) > 0 &&
             msg[i]->msg[strlen(msg[i]->msg) - 1] != '\n')
            fputc('\n', stderr);
         break;
                case PAM_TEXT_INFO:
         fputs(msg[i]->msg, stdout);
         if (strlen(msg[i]->msg) > 0 &&
             msg[i]->msg[strlen(msg[i]->msg) - 1] != '\n')
            fputc('\n', stdout);
            break;
         default:
            return PAM_CONV_ERR;
      }
        }
   *resp = aresp;
   return PAM_SUCCESS;
}

struct pam_conv conv = { conv_func, NULL };

static void strstrip(char *str, int strlen)
{
   int i;
   for (i = 0; i < strlen; ++i) {
      if (str[i] == '\n') {
         str[i] = 0;
         break;
      }
   }
}

int main(int argc, char **argv)
{
   char user[1024];
   pam_handle_t *test_pamh = NULL;
   char **envlist;
   int ret;

   printf("User: ");
   if (!fgets(user, sizeof(user), stdin)) {
      printf("Bad user.\n");
      return 1;
   }

   strstrip(user, sizeof(user));
   
   ret = pam_start("pam_kwallet_test", user, &conv, &test_pamh);
   if (ret != PAM_SUCCESS) {
      printf("Error initializing pam.");
      return 1;
   }

   pam_set_item(test_pamh, PAM_XDISPLAY, (const void*)":0");

   ret = pam_authenticate(test_pamh, 0);
   if (ret != PAM_SUCCESS) {
      printf("Bad user/password?\n");
      return 1;
   }

   ret = pam_open_session(test_pamh, 0);
   if (ret != PAM_SUCCESS) {
      printf("Opening the session failed.\n");
      return 1;
   }

   envlist = pam_getenvlist(test_pamh);
   while (*envlist != NULL) {
      printf("%s\n", *envlist);
      free(*envlist);
   }
   free(envlist);

   ret = pam_close_session(test_pamh, 0);
   if (ret != PAM_SUCCESS) {
      printf("Closing the session failed.\n");
      return 1;
   }

   pam_end(test_pamh, PAM_SUCCESS);

   return 0;
}
