/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef CONTACTS_MARBLE_WIDGET_H_
#define CONTACTS_MARBLE_WIDGET_H_

#include <QtGui/QGraphicsWidget>
#include <QtGui/QGraphicsProxyWidget>
#include <QtCore/QList>

#include "annotatedmarblewidget.h"
#include "globalwidget.h"

/**
 * Displays contacts' location using marble
 */
class MarbleContactsWidget: public GlobalWidget
{
Q_OBJECT

public:
	/**
	 * Constructor
	 * @param themeID ID of the theme to load
	 * @param parent Parent widget
	 */
	explicit MarbleContactsWidget(const QString &themeID, QGraphicsItem *parent = 0)						;

	/** Destructor */
	virtual~ MarbleContactsWidget();

	/**
	 * Toggle whether a change of a contact's location should trigger a
	 * recentering of the map. Default is true.
	 * @param enabled True iff map should follow contact's location changes
	 */
	void setRecenterOnLocationChange(bool enabled);

	/**
	 * Properties of some contact have changed. Reimplement from GlobalWidget
	 * @param id Contact id
	 * @param properties New properties
	 */
	virtual void propertiesChanged(const QString &id, const QVariantMap &properties);

private:
	/** Map that is updated and shown */
	AnnotatedMarbleWidget *m_marble;

	QList<QString> m_contactsWithLocation;
};

#endif // CONTACTS_MARBLE_WIDGET_H_
