/*
# * Copyright(c) 2008 Matthew Adams <roguestar191 at comcast.net>
# *
# * This program is free software; you can redistribute it and/or
# * modify it under the terms of the GNU General Public License as
# * published by the Free Software Foundation; either version 2 of
# * the License, or(at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <http//www.gnu.org/licenses/>.
# */
#ifndef Ui_PlasmaTools
#define Ui_PlasmaTools

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QComboBox>
#include <QtGui/QDialog>
#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QLabel>
#include <QtGui/QListWidget>
#include <QtGui/QProgressBar>
#include <QtGui/QScrollArea>
#include <QtGui/QSpacerItem>
#include <QtGui/QTabWidget>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include <QtWebKit/QWebView>
#include <QGraphicsLinearLayout>
#include <QGraphicsGridLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsWidget>
#include <QGraphicsSceneMouseEvent>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QAction>
#include <QTimer>
#include <QTreeView>
#include <QWebPage>
#include <QWebFrame>
#include <QWebHistory>
#include <QListWidget>
#include <QScrollArea>

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QTimer>
#include "karrowbutton.h"
#include "kpushbutton.h"
#include "kurlrequester.h"
#include <kglobalsettings.h>
#include <khistorycombobox.h>
#include <KIcon>
#include <KCompletion>
#include <KBookmarkManager>
#include <KIconLoader>
#include <KUrlPixmapProvider>
#include <KUriFilter>
#include <KMessageBox>
#include <KConfigDialog>
#include <KHistoryComboBox>


#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/widgets/iconwidget.h>
#include <plasma/widgets/pushbutton.h>
#include <plasma/framesvg.h>
#include <plasma/widgets/label.h>
#include <plasma/applet.h>
#include <plasma/widgets/webview.h>
#include <plasma/widgets/slider.h>
#include <plasma/widgets/combobox.h>
#include <Plasma/IconWidget>
#include <Plasma/LineEdit>
#include <Plasma/Meter>
#include <Plasma/WebView>
#include <Plasma/TreeView>
#include <Plasma/Slider>

#include <vector>
#include <fstream>
using std::ifstream;
using namespace std;
#include <KDebug>
#include <iostream>
#include <string>
namespace plasma_tools {
	QString runCommand(QString cmd);
	QString intToString(int);
	class FrequencyMap {
		public:	
			QString orig;
			QString modified;
	};
	class FrequencyFileReader {
		// Move to .cpp?
		class FilePointer{
			ifstream *file;
			QString filename;
			int begin;
			public:
				QString OriginalFilename;
				int CpuNumber;
				int size;
				bool closed;
				FilePointer(QString _filename, int cpuNumber, QString originalFilename) : file(new ifstream()), filename(_filename), OriginalFilename(originalFilename), CpuNumber(cpuNumber) {
					closed = false;
					// Throw up an error message if !file->isOpen!..
					begin = file->tellg();
					file->seekg (0, ios::end);
					size = file->tellg();
				}
				FilePointer(const FilePointer &old) {
					FilePointer *ptr = const_cast<FilePointer*>(&old);
					ptr->file->close();
					ptr->closed = true;
					filename = ptr->filename;
					size = ptr->size;
					closed = false;
					CpuNumber = ptr->CpuNumber;
					OriginalFilename = ptr->OriginalFilename;
					file = new std::ifstream;
				}
				~FilePointer() {
					delete file;
				}
				QString read() {
					file->open(filename.toLocal8Bit().data());
					file->seekg (0, ios::end);
					size = file->tellg();
					if(!file->is_open() || file->fail() || size == -1) {
						if( !file->is_open() ){
							kDebug() << "ERROR, file was closed!";
						}
						if(file->fail() ) {
							kDebug() << "ERROR, file failed!";
						}
						if(size == -1) {
							kDebug() << "ERROR, size is -1!\n";
						}

						file->close();
						delete file;
						file = new ifstream(filename.toLocal8Bit().data());
						int count = 100;
						while (!file->is_open() && count != 0) {
							file->open(filename.toLocal8Bit().data());
							count--;
						}
					}
					if(!file->is_open()) {
						kDebug() << "Failed to open file after 100 tries " << filename;
						return "Fail";
					}

					std::string line; //char *buf = new char[size];
					file->seekg (0, ios::beg);
					QString ret;
					while (!file->eof() )
					{
						getline (*file,line);
						ret.append(line.c_str());
					}
					file->close();
					return ret.simplified();
				}
			};
			QList<QString> fileBases;
			std::vector<FilePointer*> files;
		public:
			FrequencyFileReader() {

			};
			~FrequencyFileReader() {
				std::vector<FilePointer*>::iterator iter = files.begin();
				for(;iter != files.end(); iter++) delete *iter;
			};
			static FrequencyFileReader *self() {
				static FrequencyFileReader reader;
				return &reader;
			};
			QString readFile(QString filename, int cpuNumber) {
				std::vector<FilePointer*>::iterator iter = files.begin();
				for(;iter != files.end(); iter++) {
					if( (*iter)->OriginalFilename == filename) {
						if(cpuNumber != (*iter)->CpuNumber) {
							FilePointer *fpold = *iter;
							delete fpold;
							files.erase(iter);
							QString rep = filename.replace("%s", intToString(cpuNumber));
							FilePointer *fp = new FilePointer(rep, cpuNumber, filename);
							files.push_back(fp);
							return files.back()->read();
						}
						return (*iter)->read();
					}
				}
				QString rep = filename.replace("%s", intToString(cpuNumber));
				FilePointer *fp = new FilePointer(rep, cpuNumber, filename);
				files.push_back(fp);
				return files.back()->read();
			};
	};

	class aPid {
		public:
			class aFp { 
				public: 
				FILE*fp; 
				int count; 
				aPid *top;
				aFp(FILE*_fp, aPid* _top);
				aFp(const aFp& old);
				~aFp();
			};
			void addFile(FILE*fp);
			void remFile(FILE*fp);
			static std::vector<aFp> files;
			bool alive;
			FILE *fp;
			QString buffer;
			aPid(FILE* _fp);
			aPid(const aPid &old);
			~aPid();
			ssize_t read();
	};

}

#endif

