/***************************************************************************
 *   Copyright (C) 2008 by Sebastian Kuegler <sebas@kde.org>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef GRID_HEADER
#define GRID_HEADER

#include <QGraphicsGridLayout>
#include <QGraphicsProxyWidget>
#include <QLabel>
#include <QPointer>

#include <Plasma/Applet>
#include <Plasma/Containment>

#include "ui_gridConfig.h"

class Grid : public Plasma::Containment
{
    Q_OBJECT
    public:
        Grid(QObject *parent, const QVariantList &args);
        ~Grid();

        void init();
        void paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option,
                            const QRect &contents);
        void constraintsEvent(Plasma::Constraints constraints);
        QSizeF contentSizeHint() const;

    public Q_SLOTS:
        void dropEvent(QGraphicsSceneDragDropEvent* event);
        void layoutApplet(Plasma::Applet* applet, const QPointF &pos);

    protected Q_SLOTS:
        void configAccepted();

    protected:
        void createConfigurationInterface(KConfigDialog *parent);


    private:
        void initLayout();
        void initApplets();
        void loadApplet (QString appletName, int n);
        void loadApplet (Plasma::Applet* applet, int n);

        int getRectNumber(QPointF pos);
        QStringList m_appletNames;
        QMap<int, QPair<int, int> > m_fields;
        QMap<int, QRectF> m_rects;
        QMap<int, QPointer<Plasma::Applet> > m_applets;
        int m_rows;
        int m_columns;

        QGraphicsGridLayout *m_layout;
        Ui::gridConfig ui;

};

K_EXPORT_PLASMA_APPLET(grid, Grid)

#endif
