/*
    Copyright (C) 2009  George Kiagiadakis <kiagiadakis.george@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "qgstcolorbalance.h"
#include <gst/interfaces/colorbalance.h>

namespace QtGstreamer {

QGstColorBalance::QGstColorBalance(const QGstElementPtr & element)
    : m_element(element)
{
}

QGstColorBalance::~QGstColorBalance()
{
}

//static
bool QGstColorBalance::isColorBalance(const QGstElementPtr & element)
{
    return GST_IS_COLOR_BALANCE(element->peekNativeObject());
}

/*QGstColorBalance::ColorBalanceType QGstColorBalance::getBalanceType()
{
    return static_cast<ColorBalanceType>(gst_color_balance_get_balance_type(GST_COLOR_BALANCE(m_element->peekNativeObject())));
}*/

QList<QGstColorBalanceChannelPtr> QGstColorBalance::listChannels() const
{
    const GList *nativeChannels = gst_color_balance_list_channels(GST_COLOR_BALANCE(m_element->peekNativeObject()));
    QList<QGstColorBalanceChannelPtr> channels;

    for (; nativeChannels != NULL; nativeChannels = nativeChannels->next) {
        GstColorBalanceChannel *c = (GstColorBalanceChannel *)nativeChannels->data;

        channels.append(QGstColorBalanceChannel::fromGstColorBalanceChannel(c));        
    }

    return channels;
}

int QGstColorBalance::getValue(const QGstColorBalanceChannelPtr &channel)
{
    return gst_color_balance_get_value(GST_COLOR_BALANCE(m_element->peekNativeObject()),GST_COLOR_BALANCE_CHANNEL(channel->peekNativeObject()));
}

void QGstColorBalance::setValue(const QGstColorBalanceChannelPtr &channel,int value)
{
    gst_color_balance_set_value(GST_COLOR_BALANCE(m_element->peekNativeObject()),GST_COLOR_BALANCE_CHANNEL(channel->peekNativeObject()),value);
}

void QGstColorBalance::valueChanged(const QGstColorBalanceChannelPtr &channel,int value)
{
    gst_color_balance_value_changed(GST_COLOR_BALANCE(m_element->peekNativeObject()),GST_COLOR_BALANCE_CHANNEL(channel->peekNativeObject()),value);
}

}
