/*
 *   Copyright 2008 by Andrea Diamantini <adjam7@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "konsolator.h"

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QGraphicsScene>
#include <QGraphicsView>

#include <plasma/svg.h>
#include <plasma/theme.h>
#include <Plasma/Corona>

#include <KDebug>
#include <kde_terminal_interface.h>
#include <klibloader.h>
#include <kshell.h>
#include <kparts/part.h>
#include <KUrl>


Konsolator::Konsolator(QObject *parent, const QVariantList &args)
    : Plasma::PopupApplet(parent, args),
      m_widget(0),
      m_terminal(0),
      m_failed(false)
{
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setMinimumSize(100,80);
}


Konsolator::~Konsolator()
{
    delete m_terminal;
}


QWidget *Konsolator::widget()
{
    if (m_widget || m_failed) {
        return m_widget;
    }

    // -- loading a shell emulator kpart..
    KPluginFactory *factory = KPluginLoader("libkonsolepart").factory();
    KParts::ReadOnlyPart *part = factory ? (factory->create<KParts::ReadOnlyPart>(this)) : 0;

    if (!part) {
        m_failed = true;
        return 0;
    }

    m_terminal = qobject_cast<TerminalInterface*>(part);

    if (!m_terminal) {
        m_failed = true;
        delete part;
        return 0;
    }

    m_terminal->showShellInDir(KUrl().path());
    m_terminal->sendInput("cd " + KShell::quoteArg(KUrl().path()) + '\n');
    m_terminal->sendInput("clear\n");
    m_widget = part->widget();
    return m_widget;
}

#include "konsolator.moc"
