/***************************************************************************
 *   Copyright (C) 2005,2006,2007 by Mario Young                           *
 *   maye.co@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

//Qt includes
#include <QPainter>
#include <QStyleOptionGraphicsItem>

//KDE includes
#include <KConfigDialog>
#include <KDebug>
#include <KLocale>
#include <KIconLoader>
#include <KMimeType>
#include <KPropertiesDialog>
#include <KRun>
#include <KSharedConfig>
#include <kstandarddirs.h>
#include <KUrl>
#include <KDialog>

#include <plasma/svg.h>


//my includes
#include "plasmamoodbar.h"
#include "amarok_moodbar.h"

PlasmaMoodBar::PlasmaMoodBar(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_file(KStandardDirs::locate("data", "moodbar/01_-_My_Fairy_King.mood"))
{
    kDebug() << "will read" << m_file << m_file.path();
    setHasConfigurationInterface(true);

    m_mood = new HMoodbar();

    //FIXME: eventually this shouldn't be here
    m_mood->readFile(m_file);


    resize(500, 100);
    qDebug( "moodbar constructor" );
}

PlasmaMoodBar::~PlasmaMoodBar()
{
    delete m_mood;
}

void PlasmaMoodBar::createConfigurationInterface(KConfigDialog *parent)
{
    QWidget *w = new QWidget;
    ui.setupUi(w);
    parent->mainWidget()->layout()->setMargin(0);
    //parent->setButtons( KDialog::Ok | KDialog::Cancel:Apply );
    parent->addPage(w, i18n("Moodbar"));

    connect( parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()) );
    connect( parent, SIGNAL(okClicked()), this, SLOT(configAccepted()) );

    //m_dialog->show();
}

void PlasmaMoodBar::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &rect)
{
    if (rect.size() != m_pixmap.size()) {
        QSize s = contentsRect().size().toSize();
        m_pixmap = m_mood->draw(s.width(), s.height());
    }

    // draw just what is needed
    QRect drawArea = rect.intersected(option->rect);
    p->drawPixmap(drawArea, m_pixmap, drawArea.translated(-rect.x(), -rect.y()));
}

void PlasmaMoodBar::configAccepted()
{
    KUrl newUrl = ui.m_file->url();

    if (newUrl.isValid() && newUrl != m_file) {
        m_file = newUrl;
        m_mood->readFile(m_file);
    }
    qDebug( "config accepted" );
}

#include "plasmamoodbar.moc"

