/***************************************************************************
 *   Copyright (C) 2008 by Matej Svejda <mata@aw-modell.at>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "peachy_icon.h"

#include <QRectF>
#include <QSizeF>
#include <QPainter>
#include <QStyleOptionGraphicsItem>
#include <QGraphicsSceneHoverEvent>

#include <KDebug>

PeachyIcon::PeachyIcon(TaskPtr task, QGraphicsWidget* parent)
	: QGraphicsWidget(parent),
	m_task(task)
{
	setAcceptsHoverEvents(true);
}

PeachyIcon::~PeachyIcon()
{

}



void PeachyIcon::setScaling(qreal scaling)
{
	kDebug() << "setScaling called with " << scaling;
	m_scaling = scaling;
}

void  PeachyIcon::setMaximumIconSize(qreal size)
{
	kDebug() << "setMaximumIconSize called with " << size;
	m_maximumIconSize = size;
	// Load an prepare the icon.
	m_icon = m_task->icon((int) m_maximumIconSize, (int) m_maximumIconSize, false);
}

qreal PeachyIcon::size() const
{
	return m_maximumIconSize * m_scaling;
}

QRectF PeachyIcon::boundingRect() const
{
	return QRectF(0, 0, m_maximumIconSize * m_scaling, m_maximumIconSize * m_scaling);
}

TaskPtr PeachyIcon::task() const
{
	return m_task;
}

void PeachyIcon::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
	kDebug() << "painting icon with size " << (m_maximumIconSize * m_scaling);
	p->setRenderHint(QPainter::SmoothPixmapTransform);
	p->setRenderHint(QPainter::Antialiasing);
	p->drawPixmap(0, 0, m_icon.scaled(m_maximumIconSize * m_scaling, m_maximumIconSize * m_scaling, Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
}

void PeachyIcon::hoverMoveEvent(QGraphicsSceneHoverEvent* e)
{
	kDebug() << "got hover";
	//if(e->pos().y() > m_maximumIconSize - m_size) {
		emit gotHover(this, (qreal) (e->pos().x() / m_maximumIconSize));
	//}
	QGraphicsWidget::hoverMoveEvent(e);
}
void PeachyIcon::mousePressEvent( QGraphicsSceneMouseEvent * event)
{
	kDebug() << "got clicked";
	m_task->toggleIconified();
}
