/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef GAME
#define GAME


// own
#include "ui_gamedialog.h"

// KDE
#include <Plasma/PopupApplet>

class GameList;
class Game : public QObject
{
    Q_OBJECT


public:
    Game(QObject *parent, const QVariantMap &data, GameList *list);
    ~Game();

    QString title() const;
    QString description() const;
    QString icon() const;
    int rating() const;
    QString desktopFile() const;
    
    QString exec() const;
    bool startX() const;
  
    void moved();
    

    bool operator<(const Game &other) const;
    static bool lessThan(const Game* g1, const Game* g2);

    
public slots:
    void setTitle(const QString &title);
    void setDescription(const QString &description);
    void setIcon(const QString &icon);
    void setRating(const int &rating);
    void setDesktopFile(const QString &file);
    
    void setExec(const QString &exec);
    void setStartX(const bool &start);
    
    void start();
    void edit();
    void remove();
  

private:
    QVariantMap m_data;
    Ui::GameDialog ui_gameDialog;
    GameList *m_list;
    
    void copyData();
    QString gameDir() const;

    
private slots: 
    void init();
    void createDesktopFile();
    void editFinished();
    void save();
    

protected:

    
signals:
    void changed();
    
    
};



#endif
