/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef GAMELIST
#define GAMELIST

// Qt
#include <QGraphicsWidget>

// KDE
#include <KUrl>


class HoverWidget;
class Game;
class AnimatedLayout;
class GameList : public QGraphicsWidget
{
    Q_OBJECT


public:
    enum SortOrder {
        SortByTitle = 1,
        SortByDescription = 2,
#ifdef NEPOMUK
        SortByRating = 3,
#endif
        NoSort = 0
    };
    GameList(QGraphicsWidget *parent);
    ~GameList();
    
    QGraphicsWidget *widgetAt(const QPointF &pos);
    int indexOf(QGraphicsWidget *widget);
    QList<Game*> games() const;
    bool isEmpty() const;
    GameList::SortOrder sort() const;
    

public slots:
    void appendGame(Game *game);
    void insertGame(const int &index, Game *game);
    void move(const QPointF &target, const QPointF &destination);
    void move(const int &from, const int &to);
    void setSortOrder(const GameList::SortOrder &sort);
    

private slots:
    void syncBorders();
    void widgetHover(const QPointF &pos);
    void remove(QGraphicsWidget *widget);
    
    void upClicked();
    void downClicked();
    void removeClicked();
    void editClicked();
    
    void gameChanged();
    
    
private:
    AnimatedLayout *m_layout;
    HoverWidget *m_hoverWidget;
    int m_moveIndex ;
    GameList::SortOrder m_sort;
    

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void wheelEvent(QGraphicsSceneWheelEvent *event);
    void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
    void hoverMoveEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
    void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
    void dragMoveEvent(QGraphicsSceneDragDropEvent *event);
    void dropEvent(QGraphicsSceneDragDropEvent *event);
    void startDrag();
    
    
signals:
    void gameDrop(const QPointF &pos, const KUrl &file);

    
};


#endif

