/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// own
#include "proxylayout.h"

// KDE
#include <Plasma/Animator>
#include <Plasma/Animation>

// Qt
#include <QPropertyAnimation>
#include <QGraphicsWidget>
#include <KDebug>


ProxyLayout::ProxyLayout(QGraphicsWidget *widget, QGraphicsLayoutItem *parent)
    : QGraphicsLayout(parent), m_widget(widget)
{

    m_geometryAnimation = Plasma::Animator::create(Plasma::Animator::GeometryAnimation);
    m_geometryAnimation->setTargetWidget(m_widget);
    m_geometryAnimation->setEasingCurve(QEasingCurve::OutBounce);
    m_geometryAnimation->setProperty("duration", 700);
    
}


ProxyLayout::~ProxyLayout()
{

    delete m_geometryAnimation;
    if (m_widget) {
        m_widget->deleteLater();
    }
    
}


int ProxyLayout::count() const
{

    return 1;

}


QGraphicsLayoutItem *ProxyLayout::itemAt(int i) const
{

    Q_UNUSED(i);
    return m_widget;

}


void ProxyLayout::removeAt(int index)
{

    Q_UNUSED(index);
    m_widget = 0;
    
}


void ProxyLayout::setGeometry(const QRectF &rect)
{

    if (!m_widget) {
        QGraphicsLayout::setGeometry(rect);
        return;
    }
    
    if (m_geometryAnimation->state() == QPropertyAnimation::Running) {
        m_geometryAnimation->stop();
    }

    m_geometryAnimation->setProperty("startGeometry", m_widget->geometry());
    m_geometryAnimation->setProperty("targetGeometry", rect);
    m_geometryAnimation->start();

    QGraphicsLayout::setGeometry(rect);

}


QSizeF ProxyLayout::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{

    if (m_widget) {
        return m_widget->effectiveSizeHint(which, constraint);
    } else {
        return QSizeF(-1, -1);
    }
    
}
