#include "svglayoutitem.h"
#include <QPainter>

namespace Plasma {

SvgLayoutItem::SvgLayoutItem(QGraphicsItem *parent)
    : QGraphicsItem(parent), QGraphicsLayoutItem(), m_boundingRect(0.0, 0.0, 40.0, 40.0)
{
}

SvgLayoutItem::~SvgLayoutItem()
{
}

QRectF SvgLayoutItem::boundingRect () const
{
    return m_boundingRect;
}

void SvgLayoutItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    qreal w = m_boundingRect.width();  // width
    qreal h = m_boundingRect.height(); // height
    qreal cw = m_boundingRect.width() - m_dblLeft - m_dblRight;  // center width
    qreal ch = m_boundingRect.height() - m_dblTop - m_dblBottom; // center height

    painter->save();
    painter->setRenderHint(QPainter::SmoothPixmapTransform);
    painter->translate(m_boundingRect.left(), m_boundingRect.top());
    m_svg.paint(painter, QRectF(0, 0, m_dblLeft, m_dblTop),                                   "topleft");
    m_svg.paint(painter, QRectF(w - m_dblRight, 0, m_dblRight, m_dblTop),                     "topright");
    m_svg.paint(painter, QRectF(0, h - m_dblBottom, m_dblLeft, m_dblBottom),                  "bottomleft");
    m_svg.paint(painter, QRectF(w - m_dblRight, h - m_dblBottom, m_dblRight, m_dblBottom),    "bottomright");
    m_svg.paint(painter, QRectF(m_dblLeft-1.0, 0, cw+2.0, m_dblTop+1.0),                      "top");
    m_svg.paint(painter, QRectF(m_dblLeft-1.0, h - m_dblBottom-1.0, cw+2.0, m_dblBottom+1.0), "bottom");
    m_svg.paint(painter, QRectF(0, m_dblTop-1.0, m_dblLeft+1.0, ch+2.0),                      "left");
    m_svg.paint(painter, QRectF(w - m_dblRight-1.0, m_dblTop-1.0, m_dblRight+1.0, ch+2.0),    "right");
    m_svg.paint(painter, QRectF(m_dblLeft, m_dblTop, cw, ch),                                 "center");
    painter->restore();
}

QSizeF SvgLayoutItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
    switch(which)
    {
    case Qt::PreferredSize:
        return m_svg.size();
    default:
        return QSizeF();
    }
}

void SvgLayoutItem::setGeometry(const QRectF &rect)
{
    m_boundingRect = rect;
}

void SvgLayoutItem::setImagePath(const QString &path)
{
    m_svg.setImagePath(path);
    m_dblTop    = m_svg.elementRect("top").height();
    m_dblBottom = m_svg.elementRect("bottom").height();
    m_dblLeft   = m_svg.elementRect("left").width();
    m_dblRight  = m_svg.elementRect("right").width();
}

}

