#include "reflectioneffect.h"

#include <QPainter>
#include <QDebug>

class ReflectionEffectPrivate
{
public:
};


ReflectionEffect::ReflectionEffect( QObject *parent )
    : QGraphicsEffect(parent)
{
    d = new ReflectionEffectPrivate;
    updateBoundingRect();
}

ReflectionEffect::~ReflectionEffect()
{
    delete d;
}

// 30% of the image
#define REFLECTION_SIZE (0.3)

QRectF ReflectionEffect::boundingRectFor(const QRectF &rect) const
{
    return rect.adjusted(0,0, 0, rect.height()*REFLECTION_SIZE);
}

void ReflectionEffect::draw(QPainter *painter)
{
    // Draw the original
    drawSource(painter);

    // Then we add the reflection...

    // Get the source
    QPoint offset;
    QPixmap src = sourcePixmap(Qt::DeviceCoordinates, &offset, QGraphicsEffect::NoPad);
    QImage srcImage = src.toImage();

    // Transparency gradient (we should precompute this in  the constructor)
    QLinearGradient gradient(QPoint(0, 0), QPoint(0, src.height()));
    gradient.setColorAt(REFLECTION_SIZE, Qt::black); // 30% of the image
    gradient.setColorAt(0, Qt::white);

    // Create mask
    QImage mask = srcImage;//.copy();
    QPainter maskPainter(&mask);
    maskPainter.setCompositionMode(QPainter::CompositionMode_Source);
    maskPainter.fillRect(mask.rect(), gradient);

    //  Create reflection
    QImage reflection = srcImage.mirrored();
    reflection.setAlphaChannel(mask);

    QTransform restoreTransform = painter->worldTransform();
    painter->setWorldTransform(QTransform());
    qDebug() << "y was " << offset.y();
    qDebug() << "image height is " << srcImage.height();
    //    offset.setY( offset.y() + srcImage.height() );
    offset.setY( offset.y()+(srcImage.height()-12)/(1.0+REFLECTION_SIZE)+5 );
    qDebug() << "y is now " << offset.y();
    painter->drawImage( offset, reflection );
    painter->setWorldTransform(restoreTransform);
}


