#include "shinyicon.h"

#include <QLabel>
#include <QGraphicsProxyWidget>
#include <QGraphicsLinearLayout>
#include <QGraphicsPixmapItem>
#include <QPainter>

#include <Plasma/IconWidget>
#include <Plasma/Label>

#include "kwebthumbnailer.h"

class ShinyIconPrivate
{
public:
    KWebThumbnailer *thumbnailer;
    Plasma::IconWidget *iconWidget;
    Plasma::Label *reflection;
};

ShinyIcon::ShinyIcon(QGraphicsItem *parent, Qt::WindowFlags wFlags)
    : QGraphicsWidget(parent, wFlags)
{
    d = new ShinyIconPrivate;
   
    d->thumbnailer = new KWebThumbnailer(this);
    d->thumbnailer->setSize( QSize(96, 96) );
    connect( d->thumbnailer, SIGNAL(done(bool)), SLOT(thumbnailFinished(bool)) );

    QGraphicsLinearLayout *mainLayout = new QGraphicsLinearLayout( Qt::Vertical, this );
    setLayout(mainLayout);

    d->iconWidget = new Plasma::IconWidget(this);
    // TODO shadow and reflection

    //
    // The code below is a load of shit
    // maybe i should reuse the reflection portion of the graphics effect from before
    d->reflection =  new Plasma::Label(this);

    mainLayout->addItem( d->iconWidget );
    mainLayout->addItem( d->reflection );
}

ShinyIcon::~ShinyIcon()
{
    delete d;
}

void ShinyIcon::setUrl( const QUrl &url )
{
    d->thumbnailer->setUrl( url );
}

void ShinyIcon::start()
{
    d->thumbnailer->start();
}

void ShinyIcon::thumbnailFinished( bool ok )
{
    if (!ok) {
        kDebug() << "Unable to load thumbnail";
        return;
    }

    //
    // Setup thumbnail
    //
    QImage thumbImage = d->thumbnailer->thumbnail();
    QIcon thumb( QPixmap::fromImage(thumbImage) );
    d->iconWidget->setIcon( thumb );

    //
    // Setup reflection
    //

    // Transparency gradient
    QLinearGradient gradient(QPoint(0, 0), QPoint(0, thumbImage.height()));
    gradient.setColorAt(0.3, Qt::black); // 30% of the image
    gradient.setColorAt(0, Qt::white);

    // Create mask
    QImage mask = thumbImage;
    QPainter maskPainter(&mask);
    maskPainter.setCompositionMode(QPainter::CompositionMode_Source);
    maskPainter.fillRect(mask.rect(), gradient);

    // Create reflection
    QImage reflection = thumbImage.mirrored();
    reflection.setAlphaChannel(mask);

    d->reflection->nativeWidget()->setPixmap( QPixmap::fromImage(reflection) );
}

