#include "webbyeffect.h"

#include <QPainter>
#include <QDebug>

class WebbyEffectPrivate
{
public:
    QImage reflect;
    QPointF pos;
};


WebbyEffect::WebbyEffect(QObject *parent)
    : QGraphicsDropShadowEffect(parent)
{
    d = new WebbyEffectPrivate;
}

WebbyEffect::~WebbyEffect()
{
    delete d;
}

void WebbyEffect::setReflectionPixmap( const QPixmap &reflect )
{
    d->reflect = reflect.toImage();
}

void WebbyEffect::setReflectionImage( const QImage &reflect )
{
    d->reflect = reflect;
}

void WebbyEffect::setReflectionPosition( const QPointF &pos )
{
    d->pos = pos;
}

QRectF WebbyEffect::boundingRectFor(const QRectF &rect) const
{
    // Normal
    //    qDebug() << "webbyeffect rect" << rect;
    //    qDebug() << "rect with shadow" << QGraphicsDropShadowEffect::boundingRectFor(rect);
    //    qDebug() << "rect with reflection and shadow" << QGraphicsDropShadowEffect::boundingRectFor(rect).adjusted(0,0, 0, 0.3*d->reflect.height());
    return QGraphicsDropShadowEffect::boundingRectFor(rect).adjusted(0,0, 0, 0.3*d->reflect.height());
    // Debug
    //return rect.adjusted(0,0, 0, 0.3*d->reflect.height());
}

void WebbyEffect::draw(QPainter *painter)
{
    // Draw the original with shadow
    QGraphicsDropShadowEffect::draw(painter);
    //drawSource(painter);

    if ( d->reflect.isNull() )
        return;

    // Transparency gradient
    QLinearGradient gradient(QPoint(0, 0), QPoint(0, d->reflect.height()));
    gradient.setColorAt(REFLECTION_SIZE, Qt::black);
    gradient.setColorAt(0, Qt::white);

    // Create mask
    QImage mask = d->reflect;
    QPainter maskPainter(&mask);
    maskPainter.setCompositionMode(QPainter::CompositionMode_Source);
    maskPainter.fillRect(mask.rect(), gradient);

    // Create reflection
    QImage reflection = d->reflect.mirrored();
    reflection.setAlphaChannel(mask);

    QPointF offset = sourceBoundingRect(Qt::DeviceCoordinates).topLeft();

    QTransform restoreTransform = painter->worldTransform();
    painter->setWorldTransform(QTransform());
    painter->drawImage( offset + d->pos, reflection );
    painter->setWorldTransform(restoreTransform);
}

