/*
# * Copyright(c) 2008 Matthew Adams <roguestar191 at comcast.net>
# *
# * This program is free software; you can redistribute it and/or
# * modify it under the terms of the GNU General Public License as
# * published by the Free Software Foundation; either version 2 of
# * the License, or(at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <http//www.gnu.org/licenses/>.
# */
#ifndef UI_BROWSER_CPP
#define UI_BROWSER_CPP

#include <string>
#include <iostream>
#include "../includes/Ui_Browser.h"
#include "../includes/Ui_Dialog.h"
#include "../welcome.h"
#include <kurlpixmapprovider.h>
#include <limits.h>

#include <QtGui/QtGui>
#include <QFileInfo>
#include <QtWebKit/QtWebKit>
#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneMouseEvent>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QAction>
#include <QTimer>
#include <QTreeView>
#include <QWebView>
#include <QWebFrame>
#include <QWebHistory>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <KIcon>
#include <KCompletion>
#include <KBookmarkManager>
#include <KIconLoader>
#include <KUrlPixmapProvider>
#include <KUriFilter>
#include <KMessageBox>
#include <KConfigDialog>
#include <KHistoryComboBox>
#include <krun.h>
#include <kshell.h>
#include <kglobal.h>
#include <sys/stat.h>

#include <Plasma/IconWidget>
#include <Plasma/LineEdit>
#include <Plasma/Meter>
#include <Plasma/WebView>
#include <Plasma/TreeView>
#include <Plasma/Slider>


#include <vector>
#include "webview.h"

namespace welcome_plasmoid {
	std::vector<aPid::aFp> aPid::files;

	class aBrowserPrivate : public QObject {
		Q_OBJECT
		public:
			welcome_plasmoid::WebView *browser;
			Plasma::Slider *m_zoom;
			bool &loaded;
			int *zoom_factor;
			int &index;
			bool *canZoom;
			welcome_plasmoid::TabBar *tabbar;
			aBrowserPrivate(welcome_plasmoid::WebView *_b, Plasma::Slider *_z, int &_index, welcome_plasmoid::TabBar *tb, int *zoom, bool &load, bool *tzoom) :  loaded(load), index(_index), canZoom(tzoom){
				*canZoom = true;
				kDebug() << "creating browser private";
				m_zoom = _z;
				browser = _b;
				loaded = false;
				tabbar = tb;
				zoom_factor = zoom;
				connect(browser->page(), SIGNAL(loadProgress(int)), this, SLOT(loadProgress(int)));

			//	browser->page()->createStandardContextMenu();
				browser->page()->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
				browser->page()->setForwardUnsupportedContent(true);
				browser->page()->settings()->setAttribute(QWebSettings::PluginsEnabled, true);
			}
		public slots:
			void loadProgress(int amt) {
				if(*canZoom == false) return;
				kDebug() << *canZoom << " " << this;
				if(amt == 100) {
					m_zoom->setValue(*zoom_factor);
					if(!loaded) loaded = true;
					if(browser->page()->mainFrame()->title() != "") {
						tabbar->setTabText(index, browser->page()->mainFrame()->title());
					}
				} else {
					loaded = false;
					m_zoom->setValue(amt);
					loaded = true;
				}
			}
	};
	template <class T>
	class aBrowser {
		aBrowserPrivate *d;
		public:
			bool canZoom;
			welcome_plasmoid::WebView *browser;

			Plasma::Slider *m_zoom;
			TransparentFrame *frame;
			T* m__top;
			QObject *obj;
			int index;
			bool firstload;
			QGraphicsGridLayout *browserLayout;
			
			aBrowser(T* m_top, int _index) {
				m__top = m_top;
				index = _index;
				firstload = false;
				browser = new welcome_plasmoid::WebView();
				//QWidget *widg = new QWidget();

				m_zoom = new Plasma::Slider();
				d = new aBrowserPrivate(browser, m_zoom, index, m_top->webTabBar, &m_top->zoom_factor, m_top->webPageLoaded, &canZoom);
				obj = browser->page()->mainFrame();
				browserLayout = new QGraphicsGridLayout(NULL);
				browserLayout->addItem(m_zoom,0,0);
				browserLayout->addItem(browser,0,1);

				m_top->connect(browser->page()->mainFrame(), SIGNAL(destroyed(QObject*)), m_top, SLOT(deleteSignal(QObject*)));
				m_zoom->setMaximum(100);
				m_zoom->setMinimum(0);
				m_zoom->setOrientation(Qt::Vertical);
				m_zoom->show();
			}
			~aBrowser() {
				delete d;
			}
			void setTopBrowser(T*m_top) {
				if(!firstload) {
					firstload = true;
					browser->page()->mainFrame()->load(KUrl(m_top->cg.readEntry("Default Website", QString("http://www.kde.org/"))));
				}
				if(m_top->webBrowser() != NULL && m_top->webBrowser() != browser) disconnectTopBrowser(m_top);
				m_top->m_zoom = m_zoom;

				connectTopBrowser(m_top);

				m_top->setUrlRequesterText(browser->page()->mainFrame()->url().toString());

				if(browser->page()->history() != NULL) {
					if(browser->page()->history()->canGoBack() ) m_top->backbutton->setEnabled(true);
					else m_top->backbutton->setEnabled(false);
					if(browser->page()->history()->canGoForward() ) m_top->forwardbutton->setEnabled(true);
					else m_top->forwardbutton->setEnabled(false);
				} else {
					m_top->backbutton->setEnabled(false);
					m_top->forwardbutton->setEnabled(false);
				}
			}
			void disconnectTopBrowser(T*m_top) {
				// disconnect signals from the browser
				/*
//				try {
					m_top->webBrowser()->page()->disconnect(m_top);
					m_top->m_zoom->disconnect(m_top);
//				} catch(...) {
//					try {
//						m_top->m_zoom->disconnect(m_top);
//						m_top->webBrowser()->page()->disconnect(m_top);
//					} catch(...) {

//					}
				}
				*/
				m_top->m_zoom = NULL;
			}
			void connectTopBrowser(T*m_top) {
				m_top->connect(browser->page(), SIGNAL(loadProgress(int)), m_top, SLOT(loadProgress(int)));
				m_top->connect(m_zoom, SIGNAL(sliderMoved(int)), m_top, SLOT(zoom(int)));
				m_top->connect(m_zoom, SIGNAL(valueChanged(int)), m_top, SLOT(zoom(int)));
				m_top->connect(browser->page(), SIGNAL(linkClicked(QUrl)), m_top, SLOT(setUrlLink(QUrl)));
				m_top->connect(browser->page(), SIGNAL(downloadRequested(const QNetworkRequest&)), m_top, SLOT(downloadRequested(const QNetworkRequest&)));
//			m_top->connect(browser->page()->page(), SIGNAL(unsupportedContent(const QNetworkRequest&)), m_top, SLOT(downloadRequested(const QNetworkRequest&)));
				m_top->connect(browser->page(), SIGNAL(unsupportedContent ( QNetworkReply*)), m_top, SLOT(downloadUnsupported(QNetworkReply*)));
				m_top->connect(browser->page()->networkAccessManager(), SIGNAL(finished(QNetworkReply*)), m_top, SLOT(downloadUnsupported(QNetworkReply*)));
			}
	};

	class ArrowButton : public QGraphicsProxyWidget {
		Q_OBJECT
		public:
		KArrowButton* native;
		ArrowButton() : QGraphicsProxyWidget(NULL) {
			native = new KArrowButton();
			setWidget(native);
			native->setCheckable(true);
			native->setChecked(true);
			native->setMaximumSize(12,12);
		}
	};
	class BrowserTabPrivate : public QObject {
		Q_OBJECT
		friend class aBrowser<BrowserTabPrivate>;
		bool done; // Don't show default website on first load.
		int current_tab;
		int origMax;
		int cur_tab;
		int del_tab;
		Ui_WelcomePlasmoid *m_top;
		TipBrowser *browser;
		KUrlPixmapProvider *m_pixmapProvider;
		Url_Proxy *urlRequester;
		Plasma::IconWidget* backbutton;
		Plasma::IconWidget* forwardbutton;
		Plasma::IconWidget* favoritebut;
		Plasma::IconWidget* stopbut;
		Plasma::IconWidget* reloadbut;
		welcome_plasmoid::TabBar* webTabBar;
		ArrowButton hideWebFrame;
		welcome_plasmoid::WebView *LastwebBrowser;
		Plasma::Slider *lastm_zoom;
		
		bool destroying;
		bool isBrowserInVector(welcome_plasmoid::WebView*in) {
			std::vector<aBrowser<BrowserTabPrivate>*>::iterator iter = browserTabs.begin(), end = browserTabs.end();
			for(;iter!=end;iter++) if ((*(iter))->browser == in) return true;
			return false;
		}
		welcome_plasmoid::WebView *webBrowser() {
			if(destroying) return NULL;
			if(current_tab == 0 && adding) return NULL;
				
			if(browserTabs.size() == 0) {
				LastwebBrowser = NULL;
				if(adding) return NULL;
				addBrowser();
				return browserTabs[current_tab-1]->browser;
			}
			if(current_tab != 0){
				if( browserTabs[current_tab-1]->browser != LastwebBrowser && LastwebBrowser != NULL && isBrowserInVector(LastwebBrowser)) {
//					try {
						LastwebBrowser->disconnect(this);
						lastm_zoom->disconnect(this);
//					} catch(...) {}
					LastwebBrowser = NULL;
				}
			}
			LastwebBrowser = browserTabs[current_tab-1]->browser;
			lastm_zoom = browserTabs[current_tab-1]->m_zoom;
			return LastwebBrowser;
		}
		Plasma::Slider *m_zoom;
		bool webPageLoaded;
//		QTimer *myCommandShellTimer;
		int zoom_factor;
		std::vector<aPid> pids;
		std::vector<aBrowser<BrowserTabPrivate>*> browserTabs;
		bool isUrlLink;
		public:
		QTimer *slideTopLayoutTimer;
		QTimer *tabSwitcher;
		QTimer *delTabTimer;
		QTimer *postDelete;
		QTimer *PostAddDestroyTab;
		QString source;
		QString lasturl;
		TransparentFrame *webFrame;
		TransparentFrame *tipFrame;
		TransparentFrame *motdFrame;
		KConfigGroup cg;
		QGraphicsGridLayout *TipBrowserLayout;
		QGraphicsLinearLayout *webToolsLayout;
		QGraphicsLinearLayout *motdLayout;
		KCompletion *completion;
		bool canAddDestroy;
		bool fromDelete;
		QGraphicsWidget *topFrame;
		QGraphicsGridLayout *blankLayout;
		QGraphicsWidget *blankWidget;
		public:
		BrowserTabPrivate(Ui_WelcomePlasmoid *t, TipBrowser *b) : 	
					m_top(t), browser(b)  
		{
//init pointers
			topFrame = new QGraphicsWidget();
			blankLayout = new QGraphicsGridLayout();

			blankWidget = new QGraphicsWidget();
//			blankWidget->setMinimum(0);
//			blankWidget->setMaximum(100);
//			blankWidget->setValue(100);
//			blankWidget->setMeterType(Plasma::Meter::BarMeterHorizontal);
//
			blankLayout->addItem(blankWidget,0,0);
			blankLayout->invalidate();
			blankLayout->activate();


			topFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
			topFrame->hide();

			done = false;
			LastwebBrowser = NULL;
			fromDelete = false;
			canAddDestroy = true;
			destroying = false;
			current_tab = -1;
			cur_tab = -1;
			del_tab = -1;

			slideTopLayoutTimer = new QTimer();
			m_pixmapProvider = new KUrlPixmapProvider();

			urlRequester = new Url_Proxy(t);
			webFrame = new TransparentFrame(t);
			TipBrowserLayout = new QGraphicsGridLayout(topFrame);
			webToolsLayout = new QGraphicsLinearLayout(Qt::Horizontal, webFrame);
			webToolsLayout->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
			webFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
			webTabBar = new welcome_plasmoid::TabBar(topFrame);

			backbutton=NULL;
			forwardbutton=NULL;
			stopbut=NULL;
			reloadbut=NULL;
			webPageLoaded=false;
			zoom_factor = 30;
			source = "http://about:blank";
			lasturl = "http://about:blank";
			isUrlLink = false;			
			tabSwitcher = new QTimer();
			delTabTimer = new QTimer();
			postDelete = new QTimer();
			PostAddDestroyTab = new QTimer();
			adding = false; // adding a tab, only add one at a time.
		};
		~BrowserTabPrivate() {
			if(reloadbut != NULL) delete reloadbut;
			if(stopbut != NULL) delete stopbut;
			if(forwardbutton!= NULL) delete forwardbutton;
			if(backbutton != NULL) delete backbutton;
			delete completion;
		};
		void fillTipFiles(QComboBox *box) {
			QStringList tipfiles = runCommand("locate /tips").replace("\r", "").split('\n');
			QStringList::iterator iter = tipfiles.begin(), end = tipfiles.end();
			for(;iter != end;iter++) {
				struct stat st;
				stat((*(iter)).toLocal8Bit().data(), &st);
				if(!S_ISDIR(st.st_mode) && ((*(iter)).right(5) == ".html" || (*(iter)).right(5) == "/tips" ) ){	
					box->addItem(QApplication::translate("tOTDDialog", ("file:"+(*(iter))).toLocal8Bit().data(), NULL, QApplication::UnicodeUTF8));
				}
			}
		}
		void setupUi() {
			hidden = false;
			// First the web browser tools
			backbutton = m_top->addTool("go-previous", *webToolsLayout);
			forwardbutton = m_top->addTool("go-next", *webToolsLayout);
			reloadbut = m_top->addTool("view-refresh", *webToolsLayout);
			favoritebut = m_top->addTool("bookmark-new", *webToolsLayout);
			stopbut = m_top->addTool("process-stop", *webToolsLayout);
			backbutton->setEnabled(false);
			forwardbutton->setEnabled(false);

			stopbut->hide();
			urlRequester->url->setEditable(true);
			urlRequester->url->setDuplicatesEnabled(false);
			urlRequester->setZValue(999);
			urlRequester->url->setPixmapProvider(m_pixmapProvider);

			 cg = m_top->top->config();
			QStringList list = cg.readEntry("History list", QStringList());
			urlRequester->url->setHistoryItems(list);
			webToolsLayout->addItem(urlRequester);
			
			// Next the tip browser tools
//			aBrowser<BrowserTabPrivate> *b = new aBrowser<BrowserTabPrivate>(this, 1 ) ;
//			browserTabs.push_back(b);
			QGraphicsGridLayout *fakeLayout = new QGraphicsGridLayout();
			QGraphicsWidget *fakeItem = new QGraphicsWidget();
			fakeLayout->addItem(fakeItem,0,0);
			
			webTabBar->addTab("Some really long string to reserve some space even longer ", fakeLayout);
			webTabBar->setTabText(0, "Add New");
			//webTabBar->setCurrentIndex(0);
//			deleteTab(1);
			
			current_tab = 0;
			cur_tab = 0;
			adding = true;
//			b->setTopBrowser(this);
			adding = false;
			webTabBar->nativeWidget()->setCurrentIndex(0);
			webTabBar->nativeWidget()->setCurrentIndex(1);
			TransparentFrame *webTabFrame = new TransparentFrame(m_top);
			QGraphicsLinearLayout *webTabLayout = new QGraphicsLinearLayout(Qt::Horizontal, webTabFrame);
			webTabLayout->addItem(webTabBar);
			webTabFrame->setZValue(1);
			webTabBar->setZValue(1);
			webFrame->setZValue(999);
//			TipBrowserLayout->addItem(&hideTipFrame,0,0);
//			TipBrowserLayout->addItem(tipFrame,0,1);
			TipBrowserLayout->addItem(&hideWebFrame,0,0);
			TipBrowserLayout->addItem(webFrame,0,1);
			TipBrowserLayout->addItem(webTabFrame,1,0,1,2);

			webTabBar->nativeWidget()->setCloseButtonEnabled(true);

			// Connect the signals
			connect(webTabBar, SIGNAL(currentChanged(int)), this, SLOT(switchBrowserTab(int)));
			webTabBar->disconnect(webTabBar->nativeWidget(), SIGNAL(closeRequest(int)));
			connect(webTabBar->nativeWidget(), SIGNAL(closeRequest(int)), this, SLOT(deleteTab(int)));

			
			connect(favoritebut->action(), SIGNAL(triggered()), this, SLOT(addBookmark()));
			connect(stopbut->action(), SIGNAL(triggered()), this, SLOT(stop()));
			connect(backbutton->action(), SIGNAL(triggered()), this, SLOT(back()));
			connect(forwardbutton->action(), SIGNAL(triggered()), this, SLOT(forward()));
			connect(reloadbut->action(), SIGNAL(triggered()), this, SLOT(reload()));	


			connect(urlRequester, SIGNAL(returnPressed(QString)), this, SLOT(setCurrentUrl(QString)));
			connect(hideWebFrame.native, SIGNAL(clicked(bool)), this, SLOT(togWebControls(bool) ));


			connect(Plasma::Animator::self(), SIGNAL(animationFinished(QGraphicsItem*, Plasma::Animator::Animation)), this, SLOT(animationFinished(QGraphicsItem*, Plasma::Animator::Animation) ) );
			connect(slideTopLayoutTimer, SIGNAL(timeout()), this, SLOT(slideTopLayout()));

			connect(m_top->tabWidget->nativeWidget(), SIGNAL(currentChanged(int)), this, SLOT(mainTabChanged(int)) );

			connect(PostAddDestroyTab, SIGNAL(timeout()), this, SLOT(postAddDestroyTimeout() ));
			connect(postDelete, SIGNAL(timeout()), this, SLOT(postDeleteTimeout() ));
			connect(delTabTimer, SIGNAL(timeout()), this, SLOT(deleteTabTimeout() ));
			connect(tabSwitcher, SIGNAL(timeout()), this, SLOT(switchTabTimeout() ));

//			connect(myCommandShellTimer, SIGNAL(timeout()), this, SLOT(CommandShell()));
//			myCommandShellTimer->start(1000);

			webToolsLayout->activate();

			completion = new KCompletion();
			urlRequester->url->setCompletionObject(completion);

		}
		QString getUrl() { return webBrowser()->page()->mainFrame()->url().toString(); }
		void saveState(KConfigGroup &cg)
		{
			cg.writeEntry("Url", getUrl());
			if (urlRequester) {
				const QStringList list = urlRequester->url->historyItems();
				cg.writeEntry("History list", list);
			}
		}

		void invalidateLayouts() {}
		bool hidden;
		int slideAmount;
		void startSlide(bool show) {
			if(show) {
				if(hidden){
					hidden = false;
					slideTopLayoutTimer->start(1000/32);
				}
			} else {
				if(!hidden) {
					if(m_top->welcomeUsernameLabel->isVisible()) origMax = m_top->welcomeUsernameLabel->maximumSize().height();
					slideAmount = m_top->welcomeUsernameLabel->maximumSize().height()/8;
					hidden = true;
					slideTopLayoutTimer->start(1000/32);
				}
			}
		}
		int amt;
		bool adding;
		void addBrowser() {
			if(!adding) {
				adding = true;
				aBrowser<BrowserTabPrivate> *b = new aBrowser<BrowserTabPrivate>(this, webTabBar->count() ) ;
				browserTabs.push_back(b);
				b->setTopBrowser(this);
				webTabBar->addTab("about:blank", b->browserLayout);
				current_tab = webTabBar->count()-1;
				tabSwitcher->start(1);
				canAddDestroy = false;
				PostAddDestroyTab->start(500);
				adding = false;
			} 
		}
		void resetIndeces() {
			std::vector<aBrowser<BrowserTabPrivate>*>::iterator iter = browserTabs.begin(), end = browserTabs.end();
			for(;iter!=end;iter++) {
				if( (*(iter))->index == del_tab) {
					aBrowser<BrowserTabPrivate> *ptr = (*(iter));
					browserTabs.erase(iter);
					delete ptr;
					iter = end;
					break;
				}
			}
			iter = browserTabs.begin(), end = browserTabs.end();
			for(int x = 1;iter!=end;iter++,x++) {
				(*(iter))->index = x;
			}
			canAddDestroy = false;
			
		}
		public slots:
			void downloadProgress(qint64 a, qint64 b) {
				//blankWidget->setValue(a/b);
				std::cout << "Download progress\n";
			}
			void downloadIssueFinished() {
				std::cout << "Download finished\n";
				//blankWidget->setValue(100);
				QNetworkReply *reply = ((QNetworkReply*)sender());
				QNetworkRequest request = reply->request();
				QVariant v = request.attribute(QNetworkRequest::User);
				QString fileName = v.toString();
				QFile file(fileName);
				if (file.open(QFile::ReadWrite))
					file.write(reply->readAll());
			}
			void downloadUnsupported(QNetworkReply *reply) {
				QVariant typeHeader = reply->header(QNetworkRequest::ContentTypeHeader);

				QList<QByteArray> list = reply->rawHeaderList();

				if(list.size() == 0) return;
				QString header = typeHeader.toString();
				if(header.size() == 0) return;
//				{
////					QString acclist[] = { "html", "image", "text", "java" };
///					int c = 0;
//					for (QString iter = acclist[c]; c < 4; iter = acclist[c]) {
//						if (header.contains(iter)) { std::cout << "Iter: " << iter.toLocal8Bit().data() << " "<< "Header was: " << header.toLocal8Bit().data() << "\n"; return; }
//						c++;
//					}
//
//				}
//				if(typeHeader.toString().size() == 0) return;

				std::cout << "Download unsupported\n";
				std::cout << "Type: " << typeHeader.toString().toLocal8Bit().data() << "\n";
				std::cout << "Url: " << reply->url().toString().toLocal8Bit().data() << " Request url: " << reply->request().url().toString().toLocal8Bit().data() << "\n";
				for (int i = 0; i < list.size(); ++i) {
					std::cout << "Header part: "  << QString(list.at(i)).toLocal8Bit().data() << std::endl;
				}
				if(!header.contains("octet-stream") ) return;
//				QVariant v = reply->request().url(QNetworkRequest::User);
				
				QString defaultFileName = QFileInfo(reply->url().toString()).fileName();
				std::cout << "Default filename: " << defaultFileName.toLocal8Bit().data() << "\n";
                                QString fileName = QFileDialog::getSaveFileName(0, tr("Save File"), defaultFileName.insert(0, cg.readEntry("Default Download Folder", "~/")));
                                if (fileName.isEmpty())
                                        return;
//				QNetworkRequest request = reply->request();
  //                              QVariant v = request.attribute(QNetworkRequest::User);
    //                            QString fileName = v.toString();
                                QFile file(fileName);
                                if (file.open(QFile::ReadWrite))
                                        file.write(reply->readAll());

			}
			void downloadRequested(const QNetworkRequest& request)
			{
				std::cout << "Download requested!!!\n";
				// First prompted with a file dialog to make sure
				// they want the file and to select a download
				// location and name.
				QString defaultFileName = QFileInfo(request.url().toString()).fileName();
				QString fileName = QFileDialog::getSaveFileName(0, tr("Save File"), defaultFileName.insert(0, cg.readEntry("Default Download Folder", "~/")));
				if (fileName.isEmpty())
					return;
				// Construct a new request that stores the
				// file name that should be used when the
				// download is complete
				QNetworkRequest newRequest = request;
				newRequest.setAttribute(QNetworkRequest::User, fileName);
				// Ask the network manager to download
				// the file and connect to the progress
				// and finished signals.
				QNetworkAccessManager *networkManager = webBrowser()->page()->networkAccessManager();
				QNetworkReply *reply = networkManager->get(newRequest);
				connect( reply, SIGNAL(downloadProgress(qint64, qint64)), this, SLOT(downloadProgress(qint64, qint64)));
				connect(reply, SIGNAL(finished()), this, SLOT(downloadIssueFinished()));
			}
			void deleteSignal(QObject *obj) {
				std::vector<aBrowser<BrowserTabPrivate>*>::iterator iter = browserTabs.begin(), end = browserTabs.end();
				for(;iter!=end;iter++) {
					if ((*(iter))->obj == obj) {
						del_tab = (*(iter))->index;
						resetIndeces();
						break;
					}
				}
			}
			void postAddDestroyTimeout() {
				canAddDestroy = true;
				PostAddDestroyTab->stop();
			}
			void deleteTabTimeout() {
				std::vector<aBrowser<BrowserTabPrivate>*>::iterator iter = browserTabs.begin(), end = browserTabs.end();
				delTabTimer->stop();
				if(del_tab != -1) {
					resetIndeces();
					canAddDestroy = false;
					webTabBar->removeTab(del_tab);
					PostAddDestroyTab->start(500);
				}
				del_tab = -1;
				postDelete->start(1);
			}
			
			void postDeleteTimeout() {
				if(webTabBar->count() == 1) {
					addBrowser();
				}
				fromDelete = true;
				postDelete->stop();
			}
			void switchTabTimeout() {
				tabSwitcher->stop();
				cur_tab = current_tab;

				if(cur_tab >= webTabBar->count()) cur_tab = webTabBar->count()-1;
				webTabBar->setCurrentIndex(cur_tab);

				current_tab = cur_tab;
			}
			void switchBrowserTab(int index) {
				if(!canAddDestroy) {
					if(index == 0) if (webTabBar->count() > 1) {
						tabSwitcher->start(1);
						current_tab = 1;
					}
					return;
				}
				if(fromDelete) {
						// webTabBar->setCurrentIndex(index);
						fromDelete = false;
						return; }
				if(webTabBar->tabText(index) == "Add New" && canAddDestroy) {
					addBrowser();
					index = webTabBar->count()-1;
				} else {
					if(index == 0) { index = 1; tabSwitcher->start(100); }
					std::vector<aBrowser<BrowserTabPrivate>*>::iterator iter = browserTabs.begin(), end = browserTabs.end();
					for(;iter!=end;iter++) {
						if( (*(iter))->index == index) {
							(*(iter))->setTopBrowser(this);
							break;
						}
					}
				}
				fromDelete = false;
				current_tab = index;
			}
			void deleteTab(int index) {

				if(!canAddDestroy) return;
				if(webTabBar->tabText(index) == "Add New" || index == 0) return;

				destroying = true;
				canAddDestroy = false;

				if (webTabBar->currentIndex() == index) {
					kDebug() << "m_zoom was set to null! " << browserTabs[index-1];
					browserTabs[index-1]->canZoom = false;
					browserTabs[index-1]->browser->page()->disconnect();
					kDebug() << browserTabs[index-1] << " " << index << " " << webTabBar->currentIndex();
					LastwebBrowser = NULL;
					webTabBar->setCurrentIndex(index-1);
				}
				webTabBar->removeTab(index);
				PostAddDestroyTab->start(500);
				canAddDestroy = true;
				destroying = false;
				return;
			}
			void addBookmark() {
				QString title = webBrowser()->page()->mainFrame()->title();
				QString url = webBrowser()->page()->mainFrame()->url().toString();
				m_top->addBookmark(title, url);
				if (m_top->tabWidget->nativeWidget()->currentIndex() != 2) {
					m_top->tabWidget->setCurrentIndex(2);
					m_top->tabWidget->nativeWidget()->setCurrentIndex(0);
					m_top->tabWidget->nativeWidget()->setCurrentIndex(2);
				}

			}

			void slideTopLayout() {
				GloweyLabel *ptr = m_top->welcomeUsernameLabel;
				if(hidden) {
					// setMaximumSize(maximumSize().Width(), maximumSize().Height()-1
					if(cg.readEntry("Change Tab Animation", true)) {
						if(ptr->maximumSize().height() == origMax) {
							Plasma::Animator::self()->animateItem(m_top->welcomeUsernameLabel, Plasma::Animator::DisappearAnimation);
						}
						if(ptr->maximumSize().height() == 0) slideTopLayoutTimer->stop();
						else  {
							amt = ptr->maximumSize().height()-slideAmount;
							if(amt < 0) amt = 0;
							ptr->setMaximumSize(ptr->maximumSize().width(), amt);
						}
					} else {
						if(ptr->maximumSize().height() == 0) slideTopLayoutTimer->stop();
						else ptr->setMaximumSize(ptr->maximumSize().width(), 0);
						ptr->hide();
					} 
				} else {
					if(cg.readEntry("Change Tab Animation", true)) {
						if(ptr->maximumSize().height() == origMax) {
							Plasma::Animator::self()->animateItem(m_top->welcomeUsernameLabel, Plasma::Animator::AppearAnimation);
							slideTopLayoutTimer->stop();
						} else {
							amt = ptr->maximumSize().height()+slideAmount;
							if(amt > origMax) amt = origMax;
							ptr->setMaximumSize(ptr->maximumSize().width(), amt);
						}
					} else {
						if(ptr->maximumSize().height() == origMax) slideTopLayoutTimer->stop();
						else ptr->setMaximumSize(ptr->maximumSize().width(), origMax);
						ptr->show();
					}
				}
				m_top->gridLayout->invalidate();
				ptr->update();
			}
			void animationFinished(QGraphicsItem *item, Plasma::Animator::Animation anim) {
				if(item == m_top->welcomeUsernameLabel && anim == Plasma::Animator::DisappearAnimation){ 
					m_top->welcomeUsernameLabel->hide();
				} else if(item == m_top->welcomeUsernameLabel && anim == Plasma::Animator::AppearAnimation){ 

				}
			}
			void tabChanged(int index) {
				// 1 = browser
				if(index == 1 && m_top->welcomeUsernameLabel->isVisible() ) {
					startSlide(false);
				} else if ( !m_top->welcomeUsernameLabel->isVisible() ){
					startSlide(true);
					m_top->welcomeUsernameLabel->show();

				}
			}
			void setTipDB(QString in) {
				in.replace("file://", "");
				m_top->top->setTipDB(in);
				if(m_top->top->isHtml() == true ) {
					setCurrentUrl(in);
				} else {
					webBrowser()->page()->mainFrame()->setHtml(m_top->top->getTip());
				}
			}
			void stop() {
				if(!webBrowser()->page()) return;
				webBrowser()->page()->action(QWebPage::Stop)->trigger();
			};
			void back() {
				if(webBrowser()->page()->history() == NULL) return;
				if(webBrowser()->page()->history()->canGoBack())webBrowser()->page()->history()->back();
			};
			void forward() {
				if(webBrowser()->page()->history() == NULL) return;
				if(webBrowser()->page()->history()->canGoForward()) webBrowser()->page()->history()->forward();
			};
			void reload() {webBrowser()->page()->mainFrame()->load(KUrl(webBrowser()->page()->mainFrame()->url().toString()));};
			void nextTip() {
				webBrowser()->page()->mainFrame()->setHtml(m_top->top->nextTip());
			};
			void prevTip() {
				webBrowser()->page()->mainFrame()->setHtml(m_top->top->prevTip());
			};
			void clearUrl() { webBrowser()->page()->mainFrame()->load(KUrl("")); };
			void setUrl(QString in) {webBrowser()->page()->mainFrame()->load(KUrl(in)); };
			void linkClicked(QUrl in) {	
				webBrowser()->page()->mainFrame()->load(KUrl(in.toString()));
			}
			void setUrlRequesterText(QString in) { 
				if(in.indexOf("http://about:blank") != 0 && in.indexOf("about:blank") != 0)
					urlRequester->url->setEditUrl(KUrl(in));
			};
			void setUrlRequester(KUrl in) {setUrlRequesterText(in.prettyUrl());};
			void loadProgress(int amt) { 

				if(amt == 100 && browserTabs.size() > 0)  {
					webPageLoaded=true;
					stopbut->hide();
					reloadbut->show();

					if(webBrowser()->page()->history()->canGoBack() ) backbutton->setEnabled(true);
					else backbutton->setEnabled(false);
					if(webBrowser()->page()->history()->canGoForward() ) forwardbutton->setEnabled(true);
					else forwardbutton->setEnabled(false);
					if(webBrowser()->page()->mainFrame()->url().toString() == "about:blank" || webBrowser()->page()->mainFrame()->url().toString() == "http://about::blank") return;

					addUrlToHistory(KUrl(webBrowser()->page()->mainFrame()->url().toString()));
					
//Bugged: move to aBrowserPrivate to fix.
					QString html = webBrowser()->page()->currentFrame()->toHtml();
					if( html.contains("&lt;%icon&gt;") == true || html.contains("&lt;%file&gt;") == true ) {
						html = html.replace("&lt;%icon&gt;", ((Ui_WelcomePlasmoid*)browser)->welcomeUsernameLabel->iconpath);
						html = html.replace("&lt;%file&gt;", ((Ui_WelcomePlasmoid*)browser)->welcomeUsernameLabel->filepath);
						webBrowser()->page()->mainFrame()->setHtml(html, KUrl(webBrowser()->page()->mainFrame()->url().toString()));
					}

				} else { 
					webPageLoaded = false;
					stopbut->show();
					reloadbut->hide();
				}
			};
			void zoom(int amt) { 
				if (webPageLoaded == true) {
					zoom_factor = amt;
					webBrowser()->page()->mainFrame()->setZoomFactor(0.2+amt/20); // Qt d4.5
//					webBrowser()->page()->mainFrame()->setTextSizeMultiplier(0.2+amt/20); // Qt 4.4
				}
			};
			void togWebControls(bool in) {
				if(in) {
					webFrame->setMaximumHeight(9999);
					webFrame->show();
					hideWebFrame.native->setArrowType(Qt::UpArrow);
				} else {
					webFrame->setMaximumHeight(0);
					webFrame->hide();
					hideWebFrame.native->setArrowType(Qt::DownArrow);
				}
				webToolsLayout->invalidate();
			};

			void addUrlToHistory(KUrl in) {
				if(in.prettyUrl() != "about:blank" && in.prettyUrl() != "http://about:blank" && in.prettyUrl() != "") { 
					urlRequester->url->addToHistory(in.prettyUrl());
					urlRequester->url->setCurrentIndex(0);
					//if(done){
						if ( m_top->tabWidget->nativeWidget()->currentIndex() != 1) {
							m_top->tabWidget->setCurrentIndex(1);
							m_top->tabWidget->nativeWidget()->setCurrentIndex(0);
							m_top->tabWidget->nativeWidget()->setCurrentIndex(1);
						}
				//	} else if(in.prettyUrl().compare( m_top->top->config().readEntry("Default Website", QString("http://www.kde.org/")))) {
				//		done = true;
				//	}
				} 
				source = in.prettyUrl();
				lasturl = in.prettyUrl();
			}
			void setUrlLink(QUrl in) {
				// links from web browser
				std::cout << "Link clicked!\n";
				isUrlLink = true;
				//if(in.toString().indexOf("help:/") == 0) {
					setCurrentUrl( in.toString() );		
				//}
			}
			
			void CommandShell() {
				return; // This function is no longer used
				std::vector<aPid>::iterator iter = pids.begin(), end = pids.end();
				std::vector<std::vector<aPid>::iterator> remList;
				for(;iter != end;iter ++) {
//					try {
						ssize_t ret = (*(iter)).read();
						if(ret != -1 && (*(iter)).alive == true) {	

							webBrowser()->page()->mainFrame()->setHtml((*(iter)).buffer);
						} else {
							remList.push_back(iter);
							
						}
//					} 
//					catch(...) {
						remList.push_back(iter);
//					}
				}
				std::vector<std::vector<aPid>::iterator>::iterator iter2 = remList.begin(), end2 = remList.end();
				for(;iter2 != end2;iter2++ ) {
					pids.erase((*(iter2)));
				}
			}	
			void setCurrentUrl(QString in) {
				if(in.indexOf("http://about:blank") == 0 || in.indexOf("about:blank") == 0) return;
				if(in == "") return;
				bool urlUpdated = false;
				
				if(in.contains(":/") == false && isUrlLink == false) {
					bool containsHttp = false;
					std::string possibles[] = {"www.", ".com", ".org", ".gov", ".edu"};
					for(int x = 0; x != 5; x++ ){
						if(in.contains(possibles[x].c_str()) ) {
							in.insert(0,"http://");
							containsHttp=true;
							break;
						}
					} 
					if( in.contains("/") == false ) {
						// Non filesystem requests that don't have www, com, org, gov, edu, etc (List should be expandable later..
						// go to search provier.. for now.. just "http://www.google.com/search?q=%s" % ( in.replace(" ", "+") );
						QString searchstring = "http://www.google.com/search?hl=";
						searchstring.append(KGlobal::locale()->language());
						searchstring.append("&q=");
						in.replace(" ", "+");
						in.insert(0, searchstring);
						containsHttp = true;
						urlUpdated = true;
					}
					if(!containsHttp)
						in.insert(0,"command:/");
					urlUpdated = true;
				}

				if(in.indexOf("/") == 0 ) {
					setNextUrl(in);
				} else if(in.indexOf("file:/") == 0 ){
					setNextUrl(in);
				} else if (in.indexOf("help:/") == 0) {
					QStringList args;
					args << "khelpcenter";
					args << in;
					KRun::runCommand(KShell::joinArgs(args),0);
				} else if (in.indexOf("http://") == 0 ){
					setNextUrl(in);
				} else if (in.indexOf("command:/") == 0 && !isUrlLink){

					QString cmd = in;
					cmd.replace("command:/", "");
					QStringList args = cmd.split(" ");
					
					//	args << (*(iter));
					KRun::runCommand(KShell::joinArgs(args),0);

//					pids.push_back( popen(cmd.toLocal8Bit().data(), "r") );

				} else {
					setNextUrl(in);
					// Unknown url
				}
				//if(urlUpdated == true) {
					if(urlRequester->url->currentText() != in) {
						setUrlRequesterText(in);
					}
				//}
				
				if(isUrlLink == true) isUrlLink = false;
				timeout();
			};
			void setNextUrl(QString &in) {
				source = in;
			}
			void timeout() {
				if(webBrowser() == NULL) return;
				if(source != lasturl) {
					if(webBrowser()->page()->mainFrame()->url().toString() != source) {
						webBrowser()->page()->mainFrame()->load(KUrl(source));
					}
					lasturl = source;
					//source = lasturl;
				}
			}
		void mainTabChanged(int index) {
			static bool self = false;
			if(self == true) return;
			self = true;
			tabChanged(index);
			if(index == 1) {
				if( !cg.readEntry("Use Tab On Browser", true) ) { 
					if(m_top->tabWidget->isVisible() ) {
						m_top->gridLayout->removeItem(m_top->tabWidget); 
						m_top->tabWidget->hide();
					}	
				}

				if(!topFrame->isVisible()) {
					m_top->gridLayout->addItem( topFrame);
					topFrame->show();
					TipBrowserLayout->invalidate();
					TipBrowserLayout->activate();
				}
			} else {
				if(topFrame->isVisible()) {
					topFrame->hide();
					m_top->gridLayout->removeItem(topFrame);
				}
			}
			m_top->top->graphicsWidget()->update();
			m_top->tabChanged(index);
			m_top->gridLayout->invalidate();
			m_top->gridLayout->activate();
			self = false;
		}
	};
	TipBrowser::TipBrowser(Ui_WelcomePlasmoid *t) : Dialog_Tools() {
		priv = new BrowserTabPrivate(t, this);
	}
	TipBrowser::~TipBrowser() {
		delete (BrowserTabPrivate*)priv;
	}
	QGraphicsGridLayout* TipBrowser::createTipBrowserTab() {
		((BrowserTabPrivate*)priv)->setupUi();
		return ((BrowserTabPrivate*)priv)->blankLayout;
	}
	void TipBrowser::invalidateLayouts() {
		((BrowserTabPrivate*)priv)->invalidateLayouts();
	}
	void TipBrowser::retranslateUi() {

	}
	void TipBrowser::setBrowserCurrentUrl(QString in) {
		((BrowserTabPrivate*)priv)->setCurrentUrl(in);
	}
	void TipBrowser::fillTipFileBox(QComboBox *in) {
		((BrowserTabPrivate*)priv)->fillTipFiles(in);
	}
        void TipBrowser::Browser_mainTabChanged(int in) {
                ((BrowserTabPrivate*)priv)->mainTabChanged(in);
        }

	QGraphicsWidget *TipBrowser::getTipBrowserTab() {
		return ((BrowserTabPrivate*)priv)->topFrame;
	}
}

#include "../includes/Ui_Browser.moc"
#endif
