#include <Plasma/Corona>
#include <Plasma/Applet>
#include <QGraphicsLinearLayout>
#include <Plasma/Containment>
#include <iostream>
#include <Plasma/View>
#include <QGraphicsProxyWidget>
#include <KDebug>


class MyView : public Plasma::View {
	public:
		Plasma::Containment *containment;
		Plasma::Corona *scene;
		MyView(Plasma::Containment *_c, Plasma::Corona *s, QGraphicsView *v) : Plasma::View(_c, v), containment(_c), scene(s) {
			setAttribute(Qt::WA_NoSystemBackground);
			setWallpaperEnabled(false);
			kDebug() << isWallpaperEnabled() << " " << trackContainmentChanges();
			setWindowFlags (Qt::FramelessWindowHint);
			setStyleSheet("background: transparent"); 
		}
		void setContainment(Plasma::Containment *_c) {
			Plasma::View::setContainment(_c);
			containment = _c;
		}
		void resizeEvent(QResizeEvent *e) {
			Plasma::View::resizeEvent(e);
			if(!containment) {
				 return;
			}
			containment->setMinimumSize(e->size());
			containment->setMaximumSize(e->size());
			containment->resize(e->size());
			containment->setGeometry(geometry());
			containment->mapToView(this, geometry());
			 containment->updateConstraints(Plasma::AllConstraints);
			containment->flushPendingConstraintsEvents();
		}
		void moveEvent(QMoveEvent *e) {
			Plasma::View::moveEvent(e);
			if(!containment) return;
			containment->setPos(e->pos());
		}
};
class FloatPanel : public Plasma::Applet {
	Q_OBJECT
	Plasma::Containment *containment;
	public:
		~FloatPanel() {
			delete containment;
		}
		FloatPanel(QObject *parent, const QVariantList &args) : Applet(parent, args) {
			setHasConfigurationInterface(false);

			setAspectRatioMode(Plasma::IgnoreAspectRatio);
			QGraphicsLinearLayout *mainLayout = new QGraphicsLinearLayout(Qt::Horizontal, this);
			QGraphicsWidget *mainWidget = new QGraphicsWidget(this);
			Plasma::Corona *corona = dynamic_cast<Plasma::Corona*>(scene());//new Plasma::Corona(this->scene());
			if(!corona) {
				kDebug() << "Warning: No corona.. Making one";
				corona = new Plasma::Corona(this);
			}

			containment = corona->addContainment("welcomepanel", QVariantList());

			setAcceptDrops(true);
			containment->setAcceptDrops(true);
			containment->setMinimumSize(1,1);
			containment->setMaximumSize(167722,167722);
			containment->init();
			containment->updateConstraints(Plasma::AllConstraints);
			corona->addItem(containment);			

			QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget(mainWidget);
			QGraphicsLinearLayout *widgetLayout = new QGraphicsLinearLayout(mainWidget);
			MyView *view = new MyView(containment, corona, this->view());
			view->setScene(corona);
			proxy->setAttribute(Qt::WA_NoSystemBackground);
			setAttribute(Qt::WA_NoSystemBackground);

			widgetLayout->addItem(proxy);
			proxy->setWidget(view);
			mainLayout->addItem(mainWidget);

			kDebug() << "Containments Scene: " << containment->scene();
		}
		
};

K_EXPORT_PLASMA_APPLET(floatpanel, FloatPanel)
#include "floatpanel.moc"
