/***************************************************************************
 *   Copyright (C) 2007 by Teemu Rytilahti <tpr@d5k.net>                   *
 *   Copyright (C) 2009 by Rivo Laks <rivolaks@hot.ee>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef _ION_EMHI_H
#define _ION_EMHI_H

#include <Plasma/Weather/Ion>
#include <KIO/Job>
#include <QDateTime>
#include <QDomDocument>

class KUrl;


class KDE_EXPORT EMHIIon : public IonInterface
{
    Q_OBJECT

    public:
        EMHIIon(QObject *parent, const QVariantList &args);
        ~EMHIIon();

        void init();   // Ion setup - Implemented by Ion plugin
        void fetch();  // Get the Ion's data - Implemented by Ion plugin

        virtual bool updateIonSource(const QString &);

    public slots:
        virtual void reset();

    protected:
        void listLocations();
        void validateLocation(const QString& location);
        void updateWeatherData(const QString& location);

        /**
         * Checks if data is avilable and up-to-date and starts a fetch if it isn't.
         * @return Whether data can be used immediately (if false then a fetch was started).
         **/
        bool checkDataAvailability(int maxAge = -1);

        QStringList getLocationsList();

    protected slots:
        void processPendingRequests();

    private slots:
        void dataFetched(KIO::Job *, const QByteArray &);
        void fetchingDone(KJob *);

    private:
        KUrl mUrl;
        KIO::Job *mTransferJob;
        bool mIsFetching;

        QMap<QString, IonInterface::ConditionIcons> mConditionName2Icon;

        QByteArray mSource;
        QDomDocument mXMLDocument;
        QDateTime mFetchTime;
        
        QStringList mPendingValidationRequests;
        QStringList mPendingWeatherRequests;
        bool mPendingListRequest;
};

//K_EXPORT_PLASMA_ION(emhi, EMHIIon)
K_EXPORT_PLASMA_DATAENGINE(emhi, EMHIIon)

#endif // _ION_EMHI_H
