#include "factory.h"
#include "animation.h"
#include "engine.h"
#include "javascriptanimation_p.h"
#include <QFile>
#include <QTextStream>
#include <QDebug>

QSet<QString> Factory::s_paths;

Plasma::Animation *Factory::create(QString &path)
{
    Plasma::Animation *result = 0;
    /* Here this will be executed in the Animator::factory() and
     * the path is retrieved from the Theme class
     */
    //qDebug() << path;

    /**
     * FIXME: in future, the path and the animation name will not match 1:1
     * so we need a way to map animations to path names. trivial would be to map
     * "Zoom" to "zoom.js" (e.g. anim.toLower() + 'js') but that's also a bit lame.
     * I think we may need a mapping, much as we use KService to do for applet
     * names to libraries. Maybe we should, in fact, use KSycoca for this?
     * In any case, the method used below even allows for all anims to be defined in
     * one file!
     */
    if (!s_paths.contains(path)) {
        QFile file(path);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
            qDebug() << "failed to open js......";
            return result;
        }

        QTextStream buffer(&file);
        QString tmp(buffer.readAll());
        s_paths.insert(path);

        QScriptEngine *engine = AnimationEngine::globalEngine();
        QScriptValue def(engine->evaluate(tmp, path));
        if (engine->hasUncaughtException()) {
            const QScriptValue error = engine->uncaughtException();
            QString file = error.property("fileName").toString();
            const QString failureMsg = QString("Error in %1 on line %2.<br><br>%3").arg(
                    file).arg(error.property("lineNumber").toString()).arg(error.toString());
            qDebug() << "fail!" << failureMsg;
        }
    }
    //m_anim = new Plasma::JavascriptAnimation(this, tmp);
    //qDebug() << tmp;
    result = new Plasma::JavascriptAnimation(path);

    return result;
}
