/////////////////////////////////////////////////////////////////////////
// javascriptanimationtest.cpp                                         //
//                                                                     //
// Copyright (C)  2010  Adenilson Cavalcanti <cavalcantii@gmail.com>   //
//                                                                     //
// This library is free software; you can redistribute it and/or       //
// modify it under the terms of the GNU Lesser General Public          //
// License as published by the Free Software Foundation; either        //
// version 2.1 of the License, or (at your option) any later version.  //
//                                                                     //
// This library is distributed in the hope that it will be useful,     //
// but WITHOUT ANY WARRANTY; without even the implied warranty of      //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   //
// Lesser General Public License for more details.                     //
//                                                                     //
// You should have received a copy of the GNU Lesser General Public    //
// License along with this library; if not, write to the Free Software //
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA       //
// 02110-1301  USA                                                     //
/////////////////////////////////////////////////////////////////////////
#include <QtGui>
#include "javascriptanimation_p.h"
#include "factory.h"
#include "red.h"

QAbstractAnimation *createAnimation(QString &path, QGraphicsWidget *widget, bool connect = false)
{
    Plasma::Animation *m_anim;
    m_anim = Factory::create(path);
    if (!m_anim) {
        exit(-1);
    }
    m_anim->setTargetWidget(widget);
    if (connect) {
        QObject::connect(widget, SIGNAL(clicked()), m_anim, SLOT(start()));
    }

    return m_anim;
}

QAnimationGroup *setAnimations(QGraphicsWidget *widget)
{
    QSequentialAnimationGroup *sequential = new QSequentialAnimationGroup(widget);
    QParallelAnimationGroup *parallel = new QParallelAnimationGroup(sequential);
    /* First animation */
    QString js;
    js = "zoom.js";
    QAbstractAnimation *m_anim = createAnimation(js, widget);
    parallel->addAnimation(m_anim);

    /* Second animation */
    js = "fade.js";
    m_anim = createAnimation(js, widget);
    m_anim->setProperty("duration", 600);
    parallel->addAnimation(m_anim);
    sequential->addAnimation(parallel);

    /* Third animation */
    js = "rotation.js";
    m_anim = createAnimation(js, widget);
    m_anim->setProperty("duration", 350);
    sequential->addAnimation(m_anim);

    return sequential;
}

int main(int argc, char **argv)
{
    QApplication *app = new QApplication(argc, argv);
    QGraphicsView *view = new QGraphicsView();
    QGraphicsScene *scene = new QGraphicsScene();
    QParallelAnimationGroup *group = new QParallelAnimationGroup;
    scene->setSceneRect(0, 0, 700, 700);
    view->setScene(scene);
    RedWidget *first;
    for (int i = 0; i < 4; ++i) {
        for (int j = 0; j < 4; ++j) {
            RedWidget *widget = new RedWidget();
            if ((i == 0) && (j == 0)) {
                first = widget;
            }

            if (argc == 2) {
                widget->setIconPath(argv[1]);
            } else {
                widget->setIconPath("biggerkde.png");
            }
            scene->addItem(widget);
            //widget->setGeometry(i * 150, i * 150, 150, 150);
            widget->setGeometry(i * 150, j * 150, 150, 150);
            QAbstractAnimation *sequential = setAnimations(widget);
            QObject::connect(widget, SIGNAL(clicked()), sequential, SLOT(start()));

            group->addAnimation(sequential);
        }
    }

    QObject::connect(first, SIGNAL(clicked()), group, SLOT(start()));

    view->setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate);
    view->setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);

#if defined(Q_OS_SYMBIAN)
    view->showFullScreen();
#else
    view->show();
#endif

    return app->exec();
}
