/*
Copyright 2009 Beat Wolf <asraniel@fryx.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License or (at your option) version 3 or any later version
accepted by the membership of KDE e.V. (or its successor approved
by the membership of KDE e.V.), which shall act as a proxy
defined in Section 14 of version 3 of the license.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QObject>
#include <QTime>
#include <Eigen/Core>

#define PI 3.14159265

struct particle{
    Eigen::Vector2d p;
    Eigen::Vector2d dir;
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
};

struct gravityPoint{
    Eigen::Vector2d p;
    double force;
    double radius;
    bool limited;
};

enum Types{
    Random1,
    Random2,
    CentralStar,
    CentralStarRandom,
    CentralStarRotating,
    CentralStarRotatingCircle,
    RotatingCircles,
    ColidingGalaxies
    
};

class Engine : public QObject
{
Q_OBJECT
public:
    Engine();
    ~Engine();

    void setSize(int x, int y);
    void setParticles(int number);
    void setType(Types type);
    Types getType() const;
    
    void init();
    void moveParticles();
    
    struct particle* getParticle(int index) const;
    int particles() const;
    
    int gravityPoints() const;
    struct gravityPoint* getGravityPoint(int index) const;
    
    bool inited() const;
private:
    void resetEngine();
    
    void initRandomConditions();
    void initCentralStarConditions();
    
    
    //particles inits
    void createRandomParticlesStill();
    void createRandomParticles();
    void createRotatingParticles();
    void createRotatingParticlesCircle();
    void createRotatingParticlesCircleMultiple();
    void createCircle(Eigen::Vector2d center, int radiusA, int radiusB, int start, int end);
    
    //gravity point inits
    void createRandomGravityPoints();    
    void createCentralStarGravity();
    void createRestrictedGravityPoints();
    void createRestrictedColidingGravityPoints();
    
    int m_width;
    int m_height;
    int m_particlesCount;
    QTime m_lastUpdate;
    struct particle *m_particles;
    struct gravityPoint *m_gravityPoints;
    int m_gravityPointsCount;
    
    bool m_inited;
    
    Types m_type;
};
