/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KORATGROUP_H
#define KORATGROUP_H

#include "koratbar_export.h"
#include "koratbutton.h"
#include "koratitem.h"

#include <plasma/framesvg.h>

#include <QObject>
#include <QGraphicsWidget>
#include <QGraphicsItem>
#include <QGraphicsSceneResizeEvent>
#include <QPainter>
#include <QString>
#include <QIcon>

namespace Korat
{
class GroupPrivate;

class KORATBAR_EXPORT Group : public Item
{
    Q_OBJECT
public:
    Group(const QString &title,QGraphicsItem *parent=0);
    Group(const QIcon &icon,QGraphicsItem *parent=0);

    virtual ~Group();

    void setEnabledBorders(Plasma::FrameSvg::EnabledBorders borders);

    QString title() const;
    void setTitle(const QString &title);

    QIcon icon() const;
    void setIcon(const QIcon &icon);

    void addSeparator();
    void addItem(Item *item);

    virtual QList<WeightList> availableWidths();
    virtual qreal getWidth(const WeightList &);
    virtual void adjustWidth(const WeightList &);

Q_SIGNALS:
    void sizeChanged();

protected:
    virtual void resizeEvent(QGraphicsSceneResizeEvent * event);
    virtual void paint(QPainter *painter, 
                       const QStyleOptionGraphicsItem *option, 
                       QWidget *widget=0);

private Q_SLOTS:
    void themeUpdated();

private:
    friend class GroupPrivate;
    GroupPrivate *d;
    Q_PRIVATE_SLOT(d, void sizeUpdate())
};

} // namespace Korat

#endif // KORATGROUP_H
