/*
 *   Copyright (C) 2007  Jon de Andres Frias <jondeandres@gmail.com>
 *   Copyright (C) 2007  Christopher Blauvelt <cblauvelt@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 or later as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef PLASMA_LISTVIEW_H
#define PLASMA_LISTVIEW_H

#include <QSizeF>
#include <QPointF>
#include <QAbstractItemDelegate>
#include <QAbstractItemModel>
#include <QItemSelectionModel>
#include <QGraphicsSceneMouseEvent>
#include <QKeyEvent>

#include <plasma/widgets/widget.h>
#include <plasma/widgets/lineedit.h>



namespace Plasma
{

class PLASMA_EXPORT ListView: public Plasma::Widget
{
    Q_OBJECT

public:
    ListView(Widget *parent);
    ~ListView();

//    QSizeF sizeHint() const;

    QRectF boundingRect() const;

    void paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

    void setModel(QAbstractItemModel *model);
    QAbstractItemModel * model();
    void setItemDelegate(QAbstractItemDelegate *delegate);
    QAbstractItemDelegate * delegate();

    void setSelectionModel(QItemSelectionModel *selectionModel);
    QItemSelectionModel* selectionModel() const;

    QModelIndex indexAt(const QPointF &p) const;
    void initPaintRange();


public slots:
    void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);
    void selectionChanged(const QItemSelection &topLeft, const QItemSelection &bottomRight);
    void toUpdate();
    void scrollUp(int value);
    void scrollDown(int value);
    void scrollChanged(int value);

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);

private:
    class Private;
    Private *d;

};

}


#endif
