/*
 * Copyright 2007 Aaron Seigo <aseigo@kde.org
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef FULLVIEW_H
#define FULLVIEW_H

#include <Plasma/Corona>

#include <QGraphicsView>
#include <ActiveQt/QAxBindable>
#include <ActiveQt/QAxAggregated>

namespace Plasma
{
    class AccessAppletJob;
}

class FullView : public QGraphicsView, public QAxBindable
{
    Q_OBJECT
    Q_CLASSINFO("ClassID", "{BD210BAF-FFD4-4339-A1E4-BFFF312D01A6}");
    Q_CLASSINFO("InterfaceID", "{E992B907-7705-462c-AD31-99C7818393C3}");
    Q_CLASSINFO("EventsID", "{907D2A36-39AA-43f3-91B6-6AEA71ADBC32}");
public:
    explicit FullView(const QString &formfactor = "planar", const QString &location = "floating", QWidget *parent = 0);

    void addApplet(const QString &name, const QString& containment,
                   const QString& wallpaper, const QVariantList &args = QVariantList());

    QAxAggregated *createAggregate();

public slots:
    int   appletHeight();
    int   appletWidth();

private slots:
    void appletTransformedItself();
    void sceneRectChanged(const QRectF &rect);
    void resizeEvent(QResizeEvent *event);
    void closeEvent(QCloseEvent *event);
    void appletRemoved();
    void plasmoidAccessFinished(Plasma::AccessAppletJob *job);

private:
    Plasma::Corona m_corona;
    Plasma::FormFactor m_formfactor;
    Plasma::Location m_location;
    Plasma::Containment *m_containment;
    Plasma::Applet *m_applet;
};

#endif

