#include "scene.h"

#include "nodeitem.h"
#include "shadowitem.h"

#include "node.h"
#include "edge.h"
#include "composition.h"

#include <QGraphicsSceneMouseEvent>
#include <QMimeData>

Scene::Scene(Quasar::Composition* compo, const QRectF &sceneRect, QObject *parent)
    : QGraphicsScene(sceneRect, parent), m_composition(compo)
{
    m_currentEdge = 0;
}


void Scene::mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
    QGraphicsScene::mouseMoveEvent(mouseEvent);
    if (m_currentEdge) {
        m_currentEdge->adjust(m_currentEdge->start(),
                              mouseEvent->scenePos());
    }
}

void Scene::mousePressEvent(QGraphicsSceneMouseEvent * mouseEvent)
{
    QGraphicsScene::mousePressEvent(mouseEvent);
    //qDebug()<<"Scene::mousePressEvent "<<mouseEvent->pos();
    NodeItem *item = nodeAt(mouseEvent->scenePos());
    if (item) {
        //qDebug()<<"\t Node: " << item->node()->name();
        QString prop = item->propertyAt(
            item->mapFromScene(mouseEvent->scenePos()));
        //qDebug()<<"\t Property: " << prop;

        if (!prop.isEmpty()) {
            delete m_currentEdge;
            m_currentEdge = new Edge();
            addItem(m_currentEdge);
            m_currentEdge->setSource(item, prop);
            m_currentEdge->show();
        }
    }
}


void Scene::mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
    QGraphicsScene::mouseReleaseEvent(mouseEvent);
    NodeItem *item = nodeAt(mouseEvent->scenePos());
    if (item && m_currentEdge) {
        //qDebug()<<"\t Node: " << item->node()->name();
        QString prop = item->propertyAt(
            item->mapFromScene(mouseEvent->scenePos()));
        //qDebug()<<"\t Property: " << prop;
        if (m_currentEdge->sourceNode() == item) {
            qWarning("Can't connect node to itself!");
            delete m_currentEdge;
            m_currentEdge = 0;
        } else {
            m_currentEdge->setDest(item, prop);
            if (!m_currentEdge->updateLink()) {
                qWarning("Couldn't connect the node!");
                delete m_currentEdge;
            }
            m_currentEdge = 0;
        }
    } else {
        delete m_currentEdge;
        m_currentEdge = 0;
    }
}

NodeItem * Scene::nodeAt(const QPointF &pt) const
{
    QList<QGraphicsItem*> cs = items(pt.x(), pt.y(), 1, 1,
                                     Qt::IntersectsItemBoundingRect);
    NodeItem *item = 0;
    if (!cs.isEmpty()) {
        QGraphicsItem *qitem = cs.first();
        item = qgraphicsitem_cast<NodeItem*>(qitem);
    }
    return item;
}

void Scene::dragEnterEvent(QGraphicsSceneDragDropEvent* event)
{
    if (event->mimeData()->hasFormat("x-quasar/nodelist"))
        event->accept();
    else
        event->ignore();
}

void Scene::dragMoveEvent(QGraphicsSceneDragDropEvent* event)
{
    event->accept();
}

void Scene::dropEvent(QGraphicsSceneDragDropEvent* event)
{
    QByteArray data = event->mimeData()->data("x-quasar/nodelist");
    QDataStream stream(&data, QIODevice::ReadOnly);

    while (!stream.atEnd())
    {
        QString nodeName;
        stream >> nodeName;

        NodeItem *fitem = new NodeItem();
        fitem->setNode(m_composition->createNode(nodeName));
        fitem->setPos(event->scenePos() - QPointF(10, 10));
        addItem(fitem);
        ShadowItem *sitem = new ShadowItem(); // deleted by the NodeItem
        sitem->setShadowParent(fitem);
        fitem->setShadow(sitem);
        addItem(sitem);

        sitem->show();
        fitem->show();

        clearSelection();
        fitem->setSelected(true);
    }
}

#include "moc_scene.cpp"
