#ifndef QUASAR_NODEWALKER_H
#define QUASAR_NODEWALKER_H

#include <QList>
#include <QString>
#include <QStringList>
#include <QSharedDataPointer>

namespace Quasar
{
    class Node;

    class NodeWalker
    {
    public:
        static QList<Node*> nodesBetween(Node *start, Node *end);

    public:
        NodeWalker(Node *node=0);
        ~NodeWalker();
        NodeWalker(const NodeWalker &other);
        NodeWalker &operator=(const NodeWalker &other);

        QList<Node*> topLevelNodes() const;

        void next();
        void prev();

        bool hasNext() const;
        bool hasPrev() const;

        void moveToFirstParent();
        void moveToChild(const QString &path);
        void moveToParent(const QString &path);

        QStringList childPaths() const;
        QStringList parentPaths() const;

        Node *currentNode() const;
        Node *nextNode() const;
        Node *prevNode() const;
        Node *firstParent() const;
        Node *firstChild() const;

    private:
        class Private;
        QSharedDataPointer<Private> d;
    };
}
#endif
