#include "testnodes.h"

#include <QPainter>

ImgNode::ImgNode()
    : Node(QLatin1String("ImgNode"))
{
    Quasar::Property prop1 = { Quasar::Property::InputProperty,
                               QString::fromLatin1("image"),
                               QVariant::Image,
                               QImage(),
                               QString::fromLatin1("Blend image"),

                               QVariant(),
                               QVariant(),
                               QVariant()
    };

    Quasar::Property prop2 = { Quasar::Property::InputProperty,

                               QString::fromLatin1("opacity"),
                               QVariant::Double,
                               QImage(),
                               QString::fromLatin1("opacity of the image"),

                               0.5,
                               0,
                               1
    };

    Quasar::Property prop3 = { Quasar::Property::OutputProperty,

                               QString::fromLatin1("result"),
                               QVariant::Image,
                               QImage(),
                               QString::fromLatin1("Result node"),

                               QVariant(),
                               QVariant(),
                               QVariant(),
    };

    addProperty("number1", prop1);
    addProperty("number2", prop2);
    addProperty("result", prop3);
}

bool ImgNode::execute()
{
    QImage img = propertyValue("source1").value<QImage>();
    qreal opacity = propertyValue("source2").toDouble();
    int w = img.width(), h = img.height();
    if (img.isNull()) {
        w = 600;
        h = 400;
    }
    QImage ret(w, h, QImage::Format_ARGB32_Premultiplied);
    ret.fill(0);
    QPainter p(&ret);
    int centerX = ret.width()/2;
    int centerY = ret.height()/2;
    p.setOpacity(opacity);
    p.drawImage(centerX, centerY, img);
    p.setOpacity(1.0);
    p.fillRect(0, 0, centerX, centerY, QColor(0, 0, 0, 10));
    p.end();

    outputReady("result", QVariant(ret));
    return !ret.isNull();
}

DummyAddNode::DummyAddNode()
    : Node(QLatin1String("Dummy"))
{
    Quasar::Property prop1 = { Quasar::Property::InputProperty,

                               QString::fromLatin1("number1"),
                               QVariant::Double,
                               0,
                               QString::fromLatin1("First number"),

                               5,
                               0,
                               10
    };

    Quasar::Property prop2 = { Quasar::Property::InputProperty,

                               QString::fromLatin1("number2"),
                               QVariant::Double,
                               0,
                               QString::fromLatin1("second number"),

                               5,
                               0,
                               10
    };

    Quasar::Property prop3 = { Quasar::Property::OutputProperty,

                               QString::fromLatin1("result"),
                               QVariant::Double,
                               0,
                               QString::fromLatin1("Result node"),

                               QVariant(),
                               QVariant(),
                               QVariant(),
    };

    addProperty("number1", prop1);
    addProperty("number2", prop2);
    addProperty("result", prop3);
}

bool DummyAddNode::execute()
{
    QVariant num1 = propertyValue("number1");
    QVariant num2 = propertyValue("number2");

    qreal res = num1.toDouble() + num2.toDouble();

    outputReady("result", QVariant(res));
    return num1.isValid() && num2.isValid();
}

#include "moc_testnodes.cpp"
