
#include <QtTest/QtTest>
#include "node.h"
#include "nodewalker.h"

#include "testnodes.h"
using namespace Quasar;

class tst_Iteration : public QObject
{
    Q_OBJECT

public:
    tst_Iteration();
    virtual ~tst_Iteration();


public slots:
    void init();
    void cleanup();
private slots:
    void test_topLevel();
    void test_basics();
};


tst_Iteration::tst_Iteration()
{
}

tst_Iteration::~tst_Iteration()
{
}

void tst_Iteration::init()
{
    // No initialisation is required
}

void tst_Iteration::cleanup()
{
    // No cleanup is required.
}

void tst_Iteration::test_topLevel()
{
    DummyAddNode *dummy1 = new DummyAddNode;
    DummyAddNode *dummy2 = new DummyAddNode;
    DummyAddNode *dummy3 = new DummyAddNode;
    DummyAddNode *dummy4 = new DummyAddNode;

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy3, "result",
                          dummy4, "number1"), true);

    NodeWalker walker(dummy1);
    QCOMPARE(walker.topLevelNodes().count(), 1);
    QCOMPARE(walker.topLevelNodes().first(), dummy1);
    walker = NodeWalker(dummy2);
    QCOMPARE(walker.topLevelNodes().count(), 1);
    QCOMPARE(walker.topLevelNodes().first(), dummy1);

    walker = NodeWalker(dummy3);
    QCOMPARE(walker.topLevelNodes().count(), 1);
    QCOMPARE(walker.topLevelNodes().first(), dummy1);

    walker = NodeWalker(dummy4);
    QCOMPARE(walker.topLevelNodes().count(), 1);
    QCOMPARE(walker.topLevelNodes().first(), dummy1);
}


void tst_Iteration::test_basics()
{
    DummyAddNode *dummy1 = new DummyAddNode;
    DummyAddNode *dummy2 = new DummyAddNode;
    DummyAddNode *dummy3 = new DummyAddNode;
    DummyAddNode *dummy4 = new DummyAddNode;

    QCOMPARE(Quasar::link(dummy1, "result",
                          dummy2, "number1"), true);
    QCOMPARE(Quasar::link(dummy2, "result",
                          dummy3, "number1"), true);
    QCOMPARE(Quasar::link(dummy3, "result",
                          dummy4, "number1"), true);
#if 0
    NodeWalker walker(dummy1);
    QCOMPARE(walker.currentNode(), dummy1);
    QCOMPARE(walker.nextNode(), dummy2);
    walker.next();
    QCOMPARE(walker.currentNode(), dummy2);
    QCOMPARE(walker.nextNode(), dummy3);
    walker.next();
    QCOMPARE(walker.currentNode(), dummy3);
    QCOMPARE(walker.nextNode(), dummy4);
    walker.next();
    QCOMPARE(walker.currentNode(), dummy4);
    QCOMPARE(walker.nextNode(), (Quasar::Node*)0);
#endif
}


QTEST_APPLESS_MAIN(tst_Iteration)


#include "tst_iteration.moc"
